@@ -38,7 +38,7 @@ setmetatable(minetest.env, {
 		local func = minetest[key]
 		if type(func) == "function" then
 			rawset(table, key, function(self, ...)
-				return func(unpack({...}))
+				return func(...)
 			end)
 		else
 			rawset(table, key, nil)
@@ -46,3 +46,8 @@ setmetatable(minetest.env, {
 		return rawget(table, key)
 	end
 })
+
+function minetest.rollback_get_last_node_actor(pos, range, seconds)
+	return minetest.rollback_get_node_actions(pos, range, seconds, 1)[1]
+end
+
