@@ -712,7 +712,6 @@ end
 minetest.register_node("default:stone", {
 	description = "Stone",
 	tiles ={"default_stone.png"},
-	is_ground_content = true,
 	groups = {cracky=3},
 	drop = 'default:cobble',
 	legacy_mineral = true,
@@ -722,7 +721,6 @@ minetest.register_node("default:stone", {
 minetest.register_node("default:stone_with_coal", {
 	description = "Stone with coal",
 	tiles ={"default_stone.png^default_mineral_coal.png"},
-	is_ground_content = true,
 	groups = {cracky=3},
 	drop = 'default:coal_lump',
 	sounds = default.node_sound_stone_defaults(),
@@ -731,7 +729,6 @@ minetest.register_node("default:stone_with_coal", {
 minetest.register_node("default:stone_with_iron", {
 	description = "Stone with iron",
 	tiles ={"default_stone.png^default_mineral_iron.png"},
-	is_ground_content = true,
 	groups = {cracky=3},
 	drop = 'default:iron_lump',
 	sounds = default.node_sound_stone_defaults(),
@@ -740,8 +737,7 @@ minetest.register_node("default:stone_with_iron", {
 minetest.register_node("default:dirt_with_grass", {
 	description = "Dirt with grass",
 	tiles ={"default_grass.png", "default_dirt.png", "default_dirt.png^default_grass_side.png"},
-	is_ground_content = true,
-	groups = {crumbly=3},
+	groups = {crumbly=3, soil=1},
 	drop = 'default:dirt',
 	sounds = default.node_sound_dirt_defaults({
 		footstep = {name="default_grass_footstep", gain=0.4},
@@ -751,8 +747,7 @@ minetest.register_node("default:dirt_with_grass", {
 minetest.register_node("default:dirt_with_grass_footsteps", {
 	description = "Dirt with grass and footsteps",
 	tiles ={"default_grass_footsteps.png", "default_dirt.png", "default_dirt.png^default_grass_side.png"},
-	is_ground_content = true,
-	groups = {crumbly=3},
+	groups = {crumbly=3, soil=1},
 	drop = 'default:dirt',
 	sounds = default.node_sound_dirt_defaults({
 		footstep = {name="default_grass_footstep", gain=0.4},
@@ -762,15 +757,13 @@ minetest.register_node("default:dirt_with_grass_footsteps", {
 minetest.register_node("default:dirt", {
 	description = "Dirt",
 	tiles ={"default_dirt.png"},
-	is_ground_content = true,
-	groups = {crumbly=3},
+	groups = {crumbly=3, soil=1},
 	sounds = default.node_sound_dirt_defaults(),
 })
 
 minetest.register_node("default:sand", {
 	description = "Sand",
 	tiles ={"default_sand.png"},
-	is_ground_content = true,
 	groups = {crumbly=3, falling_node=1},
 	sounds = default.node_sound_sand_defaults(),
 })
@@ -778,7 +771,6 @@ minetest.register_node("default:sand", {
 minetest.register_node("default:gravel", {
 	description = "Gravel",
 	tiles ={"default_gravel.png"},
-	is_ground_content = true,
 	groups = {crumbly=2, falling_node=1},
 	sounds = default.node_sound_dirt_defaults({
 		footstep = {name="default_gravel_footstep", gain=0.45},
@@ -788,7 +780,6 @@ minetest.register_node("default:gravel", {
 minetest.register_node("default:sandstone", {
 	description = "Sandstone",
 	tiles ={"default_sandstone.png"},
-	is_ground_content = true,
 	groups = {crumbly=2,cracky=2},
 	drop = 'default:sand',
 	sounds = default.node_sound_stone_defaults(),
@@ -797,7 +788,6 @@ minetest.register_node("default:sandstone", {
 minetest.register_node("default:clay", {
 	description = "Clay",
 	tiles ={"default_clay.png"},
-	is_ground_content = true,
 	groups = {crumbly=3},
 	drop = 'default:clay_lump 4',
 	sounds = default.node_sound_dirt_defaults({
@@ -808,7 +798,6 @@ minetest.register_node("default:clay", {
 minetest.register_node("default:brick", {
 	description = "Brick",
 	tiles ={"default_brick.png"},
-	is_ground_content = true,
 	groups = {cracky=3},
 	drop = 'default:clay_brick 4',
 	sounds = default.node_sound_stone_defaults(),
@@ -817,7 +806,6 @@ minetest.register_node("default:brick", {
 minetest.register_node("default:tree", {
 	description = "Tree",
 	tiles ={"default_tree_top.png", "default_tree_top.png", "default_tree.png"},
-	is_ground_content = true,
 	groups = {snappy=2,choppy=2,oddly_breakable_by_hand=1},
 	sounds = default.node_sound_wood_defaults(),
 })
@@ -825,7 +813,6 @@ minetest.register_node("default:tree", {
 minetest.register_node("default:jungletree", {
 	description = "Jungle Tree",
 	tiles ={"default_jungletree_top.png", "default_jungletree_top.png", "default_jungletree.png"},
-	is_ground_content = true,
 	groups = {snappy=2,choppy=2,oddly_breakable_by_hand=1},
 	sounds = default.node_sound_wood_defaults(),
 })
@@ -849,6 +836,7 @@ minetest.register_node("default:leaves", {
 	visual_scale = 1.3,
 	tiles ={"default_leaves.png"},
 	paramtype = "light",
+	is_ground_content = false,
 	groups = {snappy=3},
 	drop = {
 		max_items = 1,
@@ -871,7 +859,6 @@ minetest.register_node("default:leaves", {
 minetest.register_node("default:cactus", {
 	description = "Cactus",
 	tiles ={"default_cactus_top.png", "default_cactus_top.png", "default_cactus_side.png"},
-	is_ground_content = true,
 	groups = {snappy=2,choppy=3},
 	sounds = default.node_sound_wood_defaults(),
 })
@@ -883,7 +870,6 @@ minetest.register_node("default:papyrus", {
 	inventory_image = "default_papyrus.png",
 	wield_image = "default_papyrus.png",
 	paramtype = "light",
-	is_ground_content = true,
 	walkable = false,
 	groups = {snappy=3},
 	sounds = default.node_sound_leaves_defaults(),
@@ -892,7 +878,6 @@ minetest.register_node("default:papyrus", {
 minetest.register_node("default:bookshelf", {
 	description = "Bookshelf",
 	tiles ={"default_wood.png", "default_wood.png", "default_bookshelf.png"},
-	is_ground_content = true,
 	groups = {snappy=2,choppy=3,oddly_breakable_by_hand=2},
 	sounds = default.node_sound_wood_defaults(),
 })
@@ -904,7 +889,6 @@ minetest.register_node("default:glass", {
 	inventory_image = minetest.inventorycube("default_glass.png"),
 	paramtype = "light",
 	sunlight_propagates = true,
-	is_ground_content = true,
 	groups = {snappy=2,cracky=3,oddly_breakable_by_hand=3},
 	sounds = default.node_sound_glass_defaults(),
 })
@@ -916,7 +900,6 @@ minetest.register_node("default:fence_wood", {
 	inventory_image = "default_fence.png",
 	wield_image = "default_fence.png",
 	paramtype = "light",
-	is_ground_content = true,
 	selection_box = {
 		type = "fixed",
 		fixed = {-1/7, -1/2, -1/7, 1/7, 1/2, 1/7},
@@ -932,7 +915,6 @@ minetest.register_node("default:rail", {
 	inventory_image = "default_rail.png",
 	wield_image = "default_rail.png",
 	paramtype = "light",
-	is_ground_content = true,
 	walkable = false,
 	selection_box = {
 		type = "fixed",
@@ -949,7 +931,6 @@ minetest.register_node("default:ladder", {
 	wield_image = "default_ladder.png",
 	paramtype = "light",
 	paramtype2 = "wallmounted",
-	is_ground_content = true,
 	walkable = false,
 	climbable = true,
 	selection_box = {
@@ -966,7 +947,6 @@ minetest.register_node("default:ladder", {
 minetest.register_node("default:wood", {
 	description = "Wood",
 	tiles ={"default_wood.png"},
-	is_ground_content = true,
 	groups = {snappy=2,choppy=2,oddly_breakable_by_hand=2},
 	sounds = default.node_sound_wood_defaults(),
 })
@@ -974,7 +954,6 @@ minetest.register_node("default:wood", {
 minetest.register_node("default:mese", {
 	description = "Mese",
 	tiles ={"default_mese.png"},
-	is_ground_content = true,
 	groups = {cracky=1,level=2},
 	sounds = default.node_sound_defaults(),
 })
@@ -982,7 +961,6 @@ minetest.register_node("default:mese", {
 minetest.register_node("default:cloud", {
 	description = "Cloud",
 	tiles ={"default_cloud.png"},
-	is_ground_content = true,
 	sounds = default.node_sound_defaults(),
 })
 
@@ -1001,6 +979,7 @@ minetest.register_node("default:water_flowing", {
 	pointable = false,
 	diggable = false,
 	buildable_to = true,
+	drowning = 1,
 	liquidtype = "flowing",
 	liquid_alternative_flowing = "default:water_flowing",
 	liquid_alternative_source = "default:water_source",
@@ -1024,6 +1003,7 @@ minetest.register_node("default:water_source", {
 	pointable = false,
 	diggable = false,
 	buildable_to = true,
+	drowning = 1,
 	liquidtype = "source",
 	liquid_alternative_flowing = "default:water_flowing",
 	liquid_alternative_source = "default:water_source",
@@ -1055,6 +1035,7 @@ minetest.register_node("default:lava_flowing", {
 	pointable = false,
 	diggable = false,
 	buildable_to = true,
+	drowning = 1,
 	liquidtype = "flowing",
 	liquid_alternative_flowing = "default:lava_flowing",
 	liquid_alternative_source = "default:lava_source",
@@ -1082,6 +1063,7 @@ minetest.register_node("default:lava_source", {
 	pointable = false,
 	diggable = false,
 	buildable_to = true,
+	drowning = 1,
 	liquidtype = "source",
 	liquid_alternative_flowing = "default:lava_flowing",
 	liquid_alternative_source = "default:lava_source",
@@ -1100,6 +1082,7 @@ minetest.register_node("default:torch", {
 	paramtype = "light",
 	paramtype2 = "wallmounted",
 	sunlight_propagates = true,
+	is_ground_content = false,
 	walkable = false,
 	light_source = LIGHT_MAX-1,
 	selection_box = {
@@ -1122,6 +1105,7 @@ minetest.register_node("default:sign_wall", {
 	paramtype = "light",
 	paramtype2 = "wallmounted",
 	sunlight_propagates = true,
+	is_ground_content = false,
 	walkable = false,
 	selection_box = {
 		type = "wallmounted",
@@ -1133,14 +1117,14 @@ minetest.register_node("default:sign_wall", {
 	legacy_wallmounted = true,
 	sounds = default.node_sound_defaults(),
 	on_construct = function(pos)
-		--local n = minetest.env:get_node(pos)
-		local meta = minetest.env:get_meta(pos)
+		--local n = minetest.get_node(pos)
+		local meta = minetest.get_meta(pos)
 		meta:set_string("formspec", "field[text;;${text}]")
 		meta:set_string("infotext", "\"\"")
 	end,
 	on_receive_fields = function(pos, formname, fields, sender)
 		--print("Sign at "..minetest.pos_to_string(pos).." got "..dump(fields))
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		fields.text = fields.text or ""
 		print((sender:get_player_name() or "").." wrote \""..fields.text..
 				"\" to sign at "..minetest.pos_to_string(pos))
@@ -1156,9 +1140,10 @@ minetest.register_node("default:chest", {
 	paramtype2 = "facedir",
 	groups = {snappy=2,choppy=2,oddly_breakable_by_hand=2},
 	legacy_facedir_simple = true,
+	is_ground_content = false,
 	sounds = default.node_sound_wood_defaults(),
 	on_construct = function(pos)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		meta:set_string("formspec",
 				"size[8,9]"..
 				"list[current_name;main;0,0;8,4;]"..
@@ -1168,7 +1153,7 @@ minetest.register_node("default:chest", {
 		inv:set_size("main", 8*4)
 	end,
 	can_dig = function(pos,player)
-		local meta = minetest.env:get_meta(pos);
+		local meta = minetest.get_meta(pos);
 		local inv = meta:get_inventory()
 		return inv:is_empty("main")
 	end,
@@ -1188,15 +1173,16 @@ minetest.register_node("default:chest_locked", {
 	paramtype2 = "facedir",
 	groups = {snappy=2,choppy=2,oddly_breakable_by_hand=2},
 	legacy_facedir_simple = true,
+	is_ground_content = false,
 	sounds = default.node_sound_wood_defaults(),
 	after_place_node = function(pos, placer)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		meta:set_string("owner", placer:get_player_name() or "")
 		meta:set_string("infotext", "Locked Chest (owned by "..
 				meta:get_string("owner")..")")
 	end,
 	on_construct = function(pos)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		meta:set_string("formspec",
 				"size[8,9]"..
 				"list[current_name;main;0,0;8,4;]"..
@@ -1207,12 +1193,12 @@ minetest.register_node("default:chest_locked", {
 		inv:set_size("main", 8*4)
 	end,
 	can_dig = function(pos,player)
-		local meta = minetest.env:get_meta(pos);
+		local meta = minetest.get_meta(pos);
 		local inv = meta:get_inventory()
 		return inv:is_empty("main")
 	end,
 	allow_metadata_inventory_move = function(pos, from_list, from_index, to_list, to_index, count, player)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		if not has_locked_chest_privilege(meta, player) then
 			minetest.log("action", player:get_player_name()..
 					" tried to access a locked chest belonging to "..
@@ -1223,7 +1209,7 @@ minetest.register_node("default:chest_locked", {
 		return count
 	end,
     allow_metadata_inventory_put = function(pos, listname, index, stack, player)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		if not has_locked_chest_privilege(meta, player) then
 			minetest.log("action", player:get_player_name()..
 					" tried to access a locked chest belonging to "..
@@ -1234,7 +1220,7 @@ minetest.register_node("default:chest_locked", {
 		return stack:get_count()
 	end,
     allow_metadata_inventory_take = function(pos, listname, index, stack, player)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		if not has_locked_chest_privilege(meta, player) then
 			minetest.log("action", player:get_player_name()..
 					" tried to access a locked chest belonging to "..
@@ -1273,9 +1259,10 @@ minetest.register_node("default:furnace", {
 	paramtype2 = "facedir",
 	groups = {cracky=2},
 	legacy_facedir_simple = true,
+	is_ground_content = false,
 	sounds = default.node_sound_stone_defaults(),
 	on_construct = function(pos)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		meta:set_string("formspec", default.furnace_inactive_formspec)
 		meta:set_string("infotext", "Furnace")
 		local inv = meta:get_inventory()
@@ -1284,7 +1271,7 @@ minetest.register_node("default:furnace", {
 		inv:set_size("dst", 4)
 	end,
 	can_dig = function(pos,player)
-		local meta = minetest.env:get_meta(pos);
+		local meta = minetest.get_meta(pos);
 		local inv = meta:get_inventory()
 		if not inv:is_empty("fuel") then
 			return false
@@ -1306,9 +1293,10 @@ minetest.register_node("default:furnace_active", {
 	drop = "default:furnace",
 	groups = {cracky=2},
 	legacy_facedir_simple = true,
+	is_ground_content = false,
 	sounds = default.node_sound_stone_defaults(),
 	on_construct = function(pos)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		meta:set_string("formspec", default.furnace_inactive_formspec)
 		meta:set_string("infotext", "Furnace");
 		local inv = meta:get_inventory()
@@ -1317,7 +1305,7 @@ minetest.register_node("default:furnace_active", {
 		inv:set_size("dst", 4)
 	end,
 	can_dig = function(pos,player)
-		local meta = minetest.env:get_meta(pos);
+		local meta = minetest.get_meta(pos);
 		local inv = meta:get_inventory()
 		if not inv:is_empty("fuel") then
 			return false
@@ -1330,18 +1318,13 @@ minetest.register_node("default:furnace_active", {
 	end,
 })
 
-function hacky_swap_node(pos,name)
-	local node = minetest.env:get_node(pos)
-	local meta = minetest.env:get_meta(pos)
-	local meta0 = meta:to_table()
+function swap_node(pos,name)
+	local node = minetest.get_node(pos)
 	if node.name == name then
 		return
 	end
 	node.name = name
-	local meta0 = meta:to_table()
-	minetest.env:set_node(pos,node)
-	meta = minetest.env:get_meta(pos)
-	meta:from_table(meta0)
+	minetest.swap_node(pos, node)
 end
 
 minetest.register_abm({
@@ -1349,7 +1332,7 @@ minetest.register_abm({
 	interval = 1.0,
 	chance = 1,
 	action = function(pos, node, active_object_count, active_object_count_wider)
-		local meta = minetest.env:get_meta(pos)
+		local meta = minetest.get_meta(pos)
 		for i, name in ipairs({
 				"fuel_totaltime",
 				"fuel_time",
@@ -1396,7 +1379,7 @@ minetest.register_abm({
 			local percent = math.floor(meta:get_float("fuel_time") /
 					meta:get_float("fuel_totaltime") * 100)
 			meta:set_string("infotext","Furnace active: "..percent.."%")
-			hacky_swap_node(pos,"default:furnace_active")
+			swap_node(pos,"default:furnace_active")
 			meta:set_string("formspec",
 				"size[8,9]"..
 				"image[2,2;1,1;default_furnace_fire_bg.png^[lowpart:"..
@@ -1422,7 +1405,7 @@ minetest.register_abm({
 
 		if fuel.time <= 0 then
 			meta:set_string("infotext","Furnace out of fuel")
-			hacky_swap_node(pos,"default:furnace")
+			swap_node(pos,"default:furnace")
 			meta:set_string("formspec", default.furnace_inactive_formspec)
 			return
 		end
@@ -1430,7 +1413,7 @@ minetest.register_abm({
 		if cooked.item:is_empty() then
 			if was_active then
 				meta:set_string("infotext","Furnace is empty")
-				hacky_swap_node(pos,"default:furnace")
+				swap_node(pos,"default:furnace")
 				meta:set_string("formspec", default.furnace_inactive_formspec)
 			end
 			return
@@ -1448,7 +1431,6 @@ minetest.register_abm({
 minetest.register_node("default:cobble", {
 	description = "Cobble",
 	tiles ={"default_cobble.png"},
-	is_ground_content = true,
 	groups = {cracky=3},
 	sounds = default.node_sound_stone_defaults(),
 })
@@ -1456,7 +1438,6 @@ minetest.register_node("default:cobble", {
 minetest.register_node("default:mossycobble", {
 	description = "Mossy Cobble",
 	tiles ={"default_mossycobble.png"},
-	is_ground_content = true,
 	groups = {cracky=3},
 	sounds = default.node_sound_stone_defaults(),
 })
@@ -1464,7 +1445,6 @@ minetest.register_node("default:mossycobble", {
 minetest.register_node("default:steelblock", {
 	description = "Steel Block",
 	tiles ={"default_steel_block.png"},
-	is_ground_content = true,
 	groups = {snappy=1,bendy=2},
 	sounds = default.node_sound_stone_defaults(),
 })
@@ -1477,6 +1457,7 @@ minetest.register_node("default:nyancat", {
 	paramtype2 = "facedir",
 	groups = {cracky=2},
 	legacy_facedir_simple = true,
+	is_ground_content = false,
 	sounds = default.node_sound_defaults(),
 })
 
@@ -1484,6 +1465,7 @@ minetest.register_node("default:nyancat_rainbow", {
 	description = "Nyancat Rainbow",
 	tiles ={"default_nc_rb.png"},
 	inventory_image = "default_nc_rb.png",
+	is_ground_content = false,
 	groups = {cracky=2},
 	sounds = default.node_sound_defaults(),
 })
@@ -1515,6 +1497,133 @@ minetest.register_node("default:apple", {
 	sounds = default.node_sound_defaults(),
 })
 
+
+local c_air = minetest.get_content_id("air")
+local c_ignore = minetest.get_content_id("ignore")
+local c_tree = minetest.get_content_id("default:tree")
+local c_leaves = minetest.get_content_id("default:leaves")
+local c_apple = minetest.get_content_id("default:apple")
+function default.grow_tree(data, a, pos, is_apple_tree, seed)
+	--[[
+		NOTE: Tree-placing code is currently duplicated in the engine
+		and in games that have saplings; both are deprecated but not
+		replaced yet
+	]]--
+	local pr = PseudoRandom(seed)
+	local th = pr:next(4, 5)
+	local x, y, z = pos.x, pos.y, pos.z
+	for yy = y, y+th-1 do
+		local vi = a:index(x, yy, z)
+		if a:contains(x, yy, z) and (data[vi] == c_air or yy == y) then
+			data[vi] = c_tree
+		end
+	end
+	y = y+th-1 -- (x, y, z) is now last piece of trunk
+	local leaves_a = VoxelArea:new{MinEdge={x=-2, y=-1, z=-2}, MaxEdge={x=2, y=2, z=2}}
+	local leaves_buffer = {}
+	
+	-- Force leaves near the trunk
+	local d = 1
+	for xi = -d, d do
+	for yi = -d, d do
+	for zi = -d, d do
+		leaves_buffer[leaves_a:index(xi, yi, zi)] = true
+	end
+	end
+	end
+	
+	-- Add leaves randomly
+	for iii = 1, 8 do
+		local d = 1
+		local xx = pr:next(leaves_a.MinEdge.x, leaves_a.MaxEdge.x - d)
+		local yy = pr:next(leaves_a.MinEdge.y, leaves_a.MaxEdge.y - d)
+		local zz = pr:next(leaves_a.MinEdge.z, leaves_a.MaxEdge.z - d)
+		
+		for xi = 0, d do
+		for yi = 0, d do
+		for zi = 0, d do
+			leaves_buffer[leaves_a:index(xx+xi, yy+yi, zz+zi)] = true
+		end
+		end
+		end
+	end
+	
+	-- Add the leaves
+	for xi = leaves_a.MinEdge.x, leaves_a.MaxEdge.x do
+	for yi = leaves_a.MinEdge.y, leaves_a.MaxEdge.y do
+	for zi = leaves_a.MinEdge.z, leaves_a.MaxEdge.z do
+		if a:contains(x+xi, y+yi, z+zi) then
+			local vi = a:index(x+xi, y+yi, z+zi)
+			if data[vi] == c_air or data[vi] == c_ignore then
+				if leaves_buffer[leaves_a:index(xi, yi, zi)] then
+					if is_apple_tree and pr:next(1, 100) <=  10 then
+						data[vi] = c_apple
+					else
+						data[vi] = c_leaves
+					end
+				end
+			end
+		end
+	end
+	end
+	end
+end
+
+minetest.register_abm({
+	nodenames = {"default:sapling"},
+	interval = 10,
+	chance = 50,
+	action = function(pos, node)
+		local is_soil = minetest.registered_nodes[minetest.get_node({x=pos.x, y=pos.y-1, z=pos.z}).name].groups.soil
+		if is_soil == nil or is_soil == 0 then return end
+		print("A sapling grows into a tree at "..minetest.pos_to_string(pos))
+		local vm = minetest.get_voxel_manip()
+		local minp, maxp = vm:read_from_map({x=pos.x-16, y=pos.y, z=pos.z-16}, {x=pos.x+16, y=pos.y+16, z=pos.z+16})
+		local a = VoxelArea:new{MinEdge=minp, MaxEdge=maxp}
+		local data = vm:get_data()
+		default.grow_tree(data, a, pos, math.random(1, 4) == 1, math.random(1,100000))
+		vm:set_data(data)
+		vm:write_to_map(data)
+		vm:update_map()
+	end
+})
+
+minetest.register_abm({
+	nodenames = {"default:dirt"},
+	interval = 2,
+	chance = 200,
+	action = function(pos, node)
+		local above = {x=pos.x, y=pos.y+1, z=pos.z}
+		local name = minetest.get_node(above).name
+		local nodedef = minetest.registered_nodes[name]
+		if nodedef and (nodedef.sunlight_propagates or nodedef.paramtype == "light")
+				and nodedef.liquidtype == "none"
+				and (minetest.get_node_light(above) or 0) >= 13 then
+			if name == "default:snow" or name == "default:snowblock" then
+				minetest.set_node(pos, {name = "default:dirt_with_snow"})
+			else
+				minetest.set_node(pos, {name = "default:dirt_with_grass"})
+			end
+		end
+	end
+})
+
+minetest.register_abm({
+	nodenames = {"default:dirt_with_grass"},
+	interval = 2,
+	chance = 20,
+	action = function(pos, node)
+		local above = {x=pos.x, y=pos.y+1, z=pos.z}
+		local name = minetest.get_node(above).name
+		local nodedef = minetest.registered_nodes[name]
+		if name ~= "ignore" and nodedef
+				and not ((nodedef.sunlight_propagates or nodedef.paramtype == "light")
+				and nodedef.liquidtype == "none") then
+			minetest.set_node(pos, {name = "default:dirt"})
+		end
+	end
+})
+
 --
 -- Crafting items
 --
@@ -1588,99 +1697,6 @@ minetest.register_alias("mapgen_stone_with_coal", "default:stone_with_coal")
 minetest.register_alias("mapgen_stone_with_iron", "default:stone_with_iron")
 minetest.register_alias("mapgen_mese", "default:mese")
 
-minetest.register_biome_groups({
-0.35, 0.10, 0.30
-})
-
---
--- Register the biomes for the map generator
---
-minetest.register_biome({
-	group_id = 0,
-	name = "Ocean",
-	terrain_type = "liquid",
-	node_top = "default:gravel",
-	node_filler = "default:stone",
-	num_top_nodes = 4,
-	height_min = -3000,
-	height_max = 3000,
-	heat_min = -20.0,
-	heat_max = 100.0,
-	humidity_min = 0.0,
-	humidity_max = 100.0,
-	scale = 10.0,
-	offset = -10.0,
-})
-
-minetest.register_biome({
-	group_id = 1,
-	name = "Beach",
-	terrain_type = "normal",
-	node_top = "default:sand",
-	node_filler = "default:stone",
-	num_top_nodes = 5,
-	height_min = -3000,
-	height_max = 3000,
-	heat_min = 5.0,
-	heat_max = 100.0,
-	humidity_min = 0.0,
-	humidity_max = 100.0,
-	scale = 5.0,
-	offset = 5.0,
-})
-
-minetest.register_biome({
-	group_id = 1,
-	name = "Gravel Beach",
-	terrain_type = "normal",
-	node_top = "default:gravel",
-	node_filler = "default:cobble",
-	num_top_nodes = 5,
-	height_min = -3000,
-	height_max = 3000,
-	heat_min = -50.0,
-	heat_max = 5.0,
-	humidity_min = 0.0,
-	humidity_max = 100.0,
-	scale = 5.0,
-	offset = 5.0,
-})
-
-minetest.register_biome({
-	group_id = 2,
-	name = "Land",
-	terrain_type = "normal",
-	node_top = "default:dirt_with_grass",
-	node_filler = "default:stone",
-	num_top_nodes = 5,
-	height_min = -3000,
-	height_max = 3000,
-	heat_min = -50.0,
-	heat_max = 100.0,
-	humidity_min = 0.0,
-	humidity_max = 100.0,
-	scale = 12.0,
-	offset = 20.0,
-})
-
-minetest.register_biome({
-	group_id = 3,
-	name = "Hills",
-	terrain_type = "normal",
-	node_top = "default:dirt",
-	node_filler = "default:stone",
-	num_top_nodes = 3,
-	height_min = -3000,
-	height_max = 3000,
-	heat_min = -50.0,
-	heat_max = 100.0,
-	humidity_min = 0.0,
-	humidity_max = 100.0,
-	scale = 60.0,
-	offset = 20.0,
-})
-
-
 -- Support old code
 function default.spawn_falling_node(p, nodename)
 	spawn_falling_node(p, nodename)
