@@ -45,14 +45,20 @@ public:
 	std::string name;
 	u32 flags;
 	
-	std::string top_nodename;
-	std::string filler_nodename;
+	std::string nname_top;
+	std::string nname_filler;
+	std::string nname_water;
+	std::string nname_dust;
+	std::string nname_dust_water;
 
 	content_t c_top;
-	s16 top_depth;
-
 	content_t c_filler;
-	s16 filler_height;
+	content_t c_water;
+	content_t c_dust;
+	content_t c_dust_water;
+	
+	s16 depth_top;
+	s16 depth_filler;
 	
 	s16 height_min;
 	s16 height_max;
@@ -84,6 +90,10 @@ public:
 
 	void addBiome(Biome *b);
 	void resolveNodeNames(INodeDefManager *ndef);
+	u8 getBiomeIdByName(const char *name);
+	
+	s16 calcBlockHeat(v3s16 p, u64 seed, float timeofday, float totaltime);
+	s16 calcBlockHumidity(v3s16 p, u64 seed, float timeofday, float totaltime);
 };
 
 #endif
