@@ -23,40 +23,29 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "connection.h"
 #include "environment.h"
 #include "irrlichttypes_extrabloated.h"
-#include "jmutex.h"
+#include "jthread/jmutex.h"
 #include <ostream>
+#include <map>
 #include <set>
 #include <vector>
 #include "clientobject.h"
 #include "gamedef.h"
 #include "inventorymanager.h"
-#include "filesys.h"
-#include "filecache.h"
 #include "localplayer.h"
-#include "server.h"
+#include "hud.h"
 #include "particles.h"
-#include "util/pointedthing.h"
-#include <algorithm>
 
 struct MeshMakeData;
 class MapBlockMesh;
-class IGameDef;
 class IWritableTextureSource;
 class IWritableShaderSource;
 class IWritableItemDefManager;
 class IWritableNodeDefManager;
 //class IWritableCraftDefManager;
-class ClientEnvironment;
+class ClientMediaDownloader;
 struct MapDrawControl;
 class MtEventManager;
-
-class ClientNotReadyException : public BaseException
-{
-public:
-	ClientNotReadyException(const char *s):
-		BaseException(s)
-	{}
-};
+struct PointedThing;
 
 struct QueuedMeshUpdate
 {
@@ -114,7 +103,7 @@ struct MeshUpdateResult
 	}
 };
 
-class MeshUpdateThread : public SimpleThread
+class MeshUpdateThread : public JThread
 {
 public:
 
@@ -132,31 +121,12 @@ public:
 	IGameDef *m_gamedef;
 };
 
-class MediaFetchThread : public SimpleThread
-{
-public:
-
-	MediaFetchThread(IGameDef *gamedef):
-		m_gamedef(gamedef)
-	{
-	}
-
-	void * Thread();
-
-	std::list<MediaRequest> m_file_requests;
-	MutexedQueue<std::pair<std::string, std::string> > m_file_data;
-	std::list<MediaRequest> m_failed;
-	std::string m_remote_url;
-	IGameDef *m_gamedef;
-};
-
 enum ClientEventType
 {
 	CE_NONE,
 	CE_PLAYER_DAMAGE,
 	CE_PLAYER_FORCE_MOVE,
 	CE_DEATHSCREEN,
-	CE_TEXTURES_UPDATED,
 	CE_SHOW_FORMSPEC,
 	CE_SPAWN_PARTICLE,
 	CE_ADD_PARTICLESPAWNER,
@@ -246,6 +216,57 @@ struct ClientEvent
 	};
 };
 
+/*
+	Packet counter
+*/
+
+class PacketCounter
+{
+public:
+	PacketCounter()
+	{
+	}
+
+	void add(u16 command)
+	{
+		std::map<u16, u16>::iterator n = m_packets.find(command);
+		if(n == m_packets.end())
+		{
+			m_packets[command] = 1;
+		}
+		else
+		{
+			n->second++;
+		}
+	}
+
+	void clear()
+	{
+		for(std::map<u16, u16>::iterator
+				i = m_packets.begin();
+				i != m_packets.end(); ++i)
+		{
+			i->second = 0;
+		}
+	}
+
+	void print(std::ostream &o)
+	{
+		for(std::map<u16, u16>::iterator
+				i = m_packets.begin();
+				i != m_packets.end(); ++i)
+		{
+			o<<"cmd "<<i->first
+					<<" count "<<i->second
+					<<std::endl;
+		}
+	}
+
+private:
+	// command, count
+	std::map<u16, u16> m_packets;
+};
+
 class Client : public con::PeerHandler, public InventoryManager, public IGameDef
 {
 public:
@@ -263,7 +284,8 @@ public:
 			IWritableItemDefManager *itemdef,
 			IWritableNodeDefManager *nodedef,
 			ISoundManager *sound,
-			MtEventManager *event
+			MtEventManager *event,
+			bool ipv6
 	);
 	
 	~Client();
@@ -305,6 +327,7 @@ public:
 	void sendChangePassword(const std::wstring oldpassword,
 			const std::wstring newpassword);
 	void sendDamage(u8 damage);
+	void sendBreath(u16 breath);
 	void sendRespawn();
 
 	ClientEnvironment& getEnv()
@@ -312,7 +335,7 @@ public:
 	
 	// Causes urgent mesh updates (unlike Map::add/removeNodeWithEvent)
 	void removeNode(v3s16 p);
-	void addNode(v3s16 p, MapNode n);
+	void addNode(v3s16 p, MapNode n, bool remove_metadata = true);
 	
 	void setPlayerControl(PlayerControl &control);
 
@@ -349,6 +372,7 @@ public:
 	void setCrack(int level, v3s16 pos);
 
 	u16 getHP();
+	u16 getBreath();
 
 	bool checkPrivilege(const std::string &priv)
 	{ return (m_privileges.count(priv) != 0); }
@@ -372,19 +396,15 @@ public:
 	std::wstring accessDeniedReason()
 	{ return m_access_denied_reason; }
 
-	float mediaReceiveProgress()
-	{
-		if (!m_media_receive_started) return 0;
-		return 1.0 * m_media_received_count / m_media_count;
-	}
-
-	bool texturesReceived()
-	{ return m_media_receive_started && m_media_received_count == m_media_count; }
 	bool itemdefReceived()
 	{ return m_itemdef_received; }
 	bool nodedefReceived()
 	{ return m_nodedef_received; }
-	
+	bool mediaReceived()
+	{ return m_media_downloader == NULL; }
+
+	float mediaReceiveProgress();
+
 	void afterContentReceived(IrrlichtDevice *device, gui::IGUIFont* font);
 
 	float getRTT(void);
@@ -401,12 +421,15 @@ public:
 	virtual bool checkLocalPrivilege(const std::string &priv)
 	{ return checkPrivilege(priv); }
 
-private:
-	
+	// The following set of functions is used by ClientMediaDownloader
 	// Insert a media file appropriately into the appropriate manager
 	bool loadMedia(const std::string &data, const std::string &filename);
+	// Send a request for conventional media transfer
+	void request_media(const std::list<std::string> &file_requests);
+	// Send a notification that no conventional media transfer is needed
+	void received_media();
 
-	void request_media(const std::list<MediaRequest> &file_requests);
+private:
 
 	// Virtual methods from con::PeerHandler
 	void peerAdded(con::Peer *peer);
@@ -416,8 +439,6 @@ private:
 	void Receive();
 	
 	void sendPlayerPos();
-	// This sends the player's current name etc to the server
-	void sendPlayerInfo();
 	// Send the item number 'item' as player item to the server
 	void sendPlayerItem(u16 item);
 	
@@ -436,7 +457,6 @@ private:
 	MtEventManager *m_event;
 
 	MeshUpdateThread m_mesh_update_thread;
-	std::list<MediaFetchThread*> m_media_fetch_threads;
 	ClientEnvironment m_env;
 	con::Connection m_con;
 	IrrlichtDevice *m_device;
@@ -462,15 +482,9 @@ private:
 	bool m_access_denied;
 	std::wstring m_access_denied_reason;
 	Queue<ClientEvent> m_client_event_queue;
-	FileCache m_media_cache;
-	// Mapping from media file name to SHA1 checksum
-	std::map<std::string, std::string> m_media_name_sha1_map;
-	bool m_media_receive_started;
-	u32 m_media_count;
-	u32 m_media_received_count;
 	bool m_itemdef_received;
 	bool m_nodedef_received;
-	friend class FarMesh;
+	ClientMediaDownloader *m_media_downloader;
 
 	// time_of_day speed approximation for old protocol
 	bool m_time_of_day_set;
