@@ -99,7 +99,6 @@ void makeCuboid(MeshCollector *collector, const aabb3f &box,
 		video::S3DVertex(min.X,min.Y,min.Z, 0,0,-1, c, txc[20],txc[23]),
 	};
 
-	v2f t;
 	for(int i = 0; i < tilecount; i++)
 				{
 				switch (tiles[i].rotation)
@@ -119,49 +118,43 @@ void makeCuboid(MeshCollector *collector, const aabb3f &box,
 						vertices[i*4+x].TCoords.rotateBy(270,irr::core::vector2df(0, 0));
 					break;
 				case 4: //FXR90
-					for (int x = 0; x < 4; x++)
+					for (int x = 0; x < 4; x++){
+						vertices[i*4+x].TCoords.X = 1.0 - vertices[i*4+x].TCoords.X;
 						vertices[i*4+x].TCoords.rotateBy(90,irr::core::vector2df(0, 0));
-
-					tiles[i].texture.pos.Y += tiles[i].texture.size.Y;
-					tiles[i].texture.size.Y *= -1;
+					}
 					break;
 				case 5: //FXR270
-					for (int x = 0; x < 4; x++)
+					for (int x = 0; x < 4; x++){
+						vertices[i*4+x].TCoords.X = 1.0 - vertices[i*4+x].TCoords.X;
 						vertices[i*4+x].TCoords.rotateBy(270,irr::core::vector2df(0, 0));
-					t=vertices[i*4].TCoords;
-					tiles[i].texture.pos.Y += tiles[i].texture.size.Y;
-					tiles[i].texture.size.Y *= -1;
+					}
 					break;
 				case 6: //FYR90
-					for (int x = 0; x < 4; x++)
+					for (int x = 0; x < 4; x++){
+						vertices[i*4+x].TCoords.Y = 1.0 - vertices[i*4+x].TCoords.Y;
 						vertices[i*4+x].TCoords.rotateBy(90,irr::core::vector2df(0, 0));
-					tiles[i].texture.pos.X += tiles[i].texture.size.X;
-					tiles[i].texture.size.X *= -1;
+					}
 					break;
 				case 7: //FYR270
-					for (int x = 0; x < 4; x++)
+					for (int x = 0; x < 4; x++){
+						vertices[i*4+x].TCoords.Y = 1.0 - vertices[i*4+x].TCoords.Y;
 						vertices[i*4+x].TCoords.rotateBy(270,irr::core::vector2df(0, 0));
-					tiles[i].texture.pos.X += tiles[i].texture.size.X;
-					tiles[i].texture.size.X *= -1;
+					}
 					break;
 				case 8: //FX
-					tiles[i].texture.pos.Y += tiles[i].texture.size.Y;
-					tiles[i].texture.size.Y *= -1;
+					for (int x = 0; x < 4; x++){
+						vertices[i*4+x].TCoords.X = 1.0 - vertices[i*4+x].TCoords.X;
+					}
 					break;
 				case 9: //FY
-					tiles[i].texture.pos.X += tiles[i].texture.size.X;
-					tiles[i].texture.size.X *= -1;
+					for (int x = 0; x < 4; x++){
+						vertices[i*4+x].TCoords.Y = 1.0 - vertices[i*4+x].TCoords.Y;
+					}
 					break;
 				default:
 					break;
 				}
 			}
-		for(s32 j=0; j<24; j++)
-	{
-		int tileindex = MYMIN(j/4, tilecount-1);
-		vertices[j].TCoords *= tiles[tileindex].texture.size;
-		vertices[j].TCoords += tiles[tileindex].texture.pos;
-	}
 	u16 indices[] = {0,1,2,2,3,0};
 	// Add to mesh collector
 	for(s32 j=0; j<24; j+=4)
@@ -218,7 +211,6 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			*/
 			TileSpec tile_liquid = f.special_tiles[0];
 			TileSpec tile_liquid_bfculled = getNodeTile(n, p, v3s16(0,0,0), data);
-			AtlasPointer &pa_liquid = tile_liquid.texture;
 
 			bool top_is_same_liquid = false;
 			MapNode ntop = data->m_vmanip.getNodeNoEx(blockpos_nodes + v3s16(x,y+1,z));
@@ -280,14 +272,10 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 
 				video::S3DVertex vertices[4] =
 				{
-					video::S3DVertex(-BS/2,0,BS/2,0,0,0, c,
-							pa_liquid.x0(), pa_liquid.y1()),
-					video::S3DVertex(BS/2,0,BS/2,0,0,0, c,
-							pa_liquid.x1(), pa_liquid.y1()),
-					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x1(), pa_liquid.y0()),
-					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x0(), pa_liquid.y0()),
+					video::S3DVertex(-BS/2,0,BS/2,0,0,0, c, 0,1),
+					video::S3DVertex(BS/2,0,BS/2,0,0,0, c, 1,1),
+					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c, 1,0),
+					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c, 0,0),
 				};
 
 				/*
@@ -359,14 +347,10 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			
 			video::S3DVertex vertices[4] =
 			{
-				video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c,
-						pa_liquid.x0(), pa_liquid.y1()),
-				video::S3DVertex(BS/2,0,BS/2, 0,0,0, c,
-						pa_liquid.x1(), pa_liquid.y1()),
-				video::S3DVertex(BS/2,0,-BS/2, 0,0,0, c,
-						pa_liquid.x1(), pa_liquid.y0()),
-				video::S3DVertex(-BS/2,0,-BS/2, 0,0,0, c,
-						pa_liquid.x0(), pa_liquid.y0()),
+				video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c, 0,1),
+				video::S3DVertex(BS/2,0,BS/2, 0,0,0, c, 1,1),
+				video::S3DVertex(BS/2,0,-BS/2, 0,0,0, c, 1,0),
+				video::S3DVertex(-BS/2,0,-BS/2, 0,0,0, c, 0,0),
 			};
 
 			v3f offset(p.X*BS, p.Y*BS + (-0.5+node_liquid_level)*BS, p.Z*BS);
@@ -386,7 +370,6 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			*/
 			TileSpec tile_liquid = f.special_tiles[0];
 			TileSpec tile_liquid_bfculled = f.special_tiles[1];
-			AtlasPointer &pa_liquid = tile_liquid.texture;
 
 			bool top_is_same_liquid = false;
 			MapNode ntop = data->m_vmanip.getNodeNoEx(blockpos_nodes + v3s16(x,y+1,z));
@@ -412,6 +395,8 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 				l = getInteriorLight(n, 0, data);
 			video::SColor c = MapBlock_LightColor(f.alpha, l, decode_light(f.light_source));
 			
+			u8 range = rangelim(nodedef->get(c_flowing).liquid_range, 1, 8);
+
 			// Neighbor liquid levels (key = relative position)
 			// Includes current node
 			std::map<v3s16, f32> neighbor_levels;
@@ -443,9 +428,14 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 
 					if(n2.getContent() == c_source)
 						level = (-0.5+node_liquid_level) * BS;
-					else if(n2.getContent() == c_flowing)
-						level = (-0.5 + ((float)(n2.param2&LIQUID_LEVEL_MASK)
-								+ 0.5) / (float)LIQUID_LEVEL_SOURCE * node_liquid_level) * BS;
+					else if(n2.getContent() == c_flowing){
+						u8 liquid_level = (n2.param2&LIQUID_LEVEL_MASK);
+						if (liquid_level <= LIQUID_LEVEL_MAX+1-range)
+							liquid_level = 0;
+						else
+							liquid_level -= (LIQUID_LEVEL_MAX+1-range);
+						level = (-0.5 + ((float)liquid_level+ 0.5) / (float)range * node_liquid_level) * BS;
+					}
 
 					// Check node above neighbor.
 					// NOTE: This doesn't get executed if neighbor
@@ -566,14 +556,10 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 				
 				video::S3DVertex vertices[4] =
 				{
-					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x0(), pa_liquid.y1()),
-					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x1(), pa_liquid.y1()),
-					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x1(), pa_liquid.y0()),
-					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x0(), pa_liquid.y0()),
+					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c, 0,1),
+					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c, 1,1),
+					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c, 1,0),
+					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c, 0,0),
 				};
 				
 				/*
@@ -647,14 +633,10 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			{
 				video::S3DVertex vertices[4] =
 				{
-					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x0(), pa_liquid.y1()),
-					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c,
-							pa_liquid.x1(), pa_liquid.y1()),
-					video::S3DVertex(BS/2,0,-BS/2, 0,0,0, c,
-							pa_liquid.x1(), pa_liquid.y0()),
-					video::S3DVertex(-BS/2,0,-BS/2, 0,0,0, c,
-							pa_liquid.x0(), pa_liquid.y0()),
+					video::S3DVertex(-BS/2,0,BS/2, 0,0,0, c, 0,1),
+					video::S3DVertex(BS/2,0,BS/2, 0,0,0, c, 1,1),
+					video::S3DVertex(BS/2,0,-BS/2, 0,0,0, c, 1,0),
+					video::S3DVertex(-BS/2,0,-BS/2, 0,0,0, c, 0,0),
 				};
 				
 				// To get backface culling right, the vertices need to go
@@ -675,48 +657,36 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 				// -Z towards +Z, thus the direction is +Z.
 				// Rotate texture to make animation go in flow direction
 				// Positive if liquid moves towards +Z
-				int dz = (corner_levels[side_corners[3][0]] +
+				f32 dz = (corner_levels[side_corners[3][0]] +
 						corner_levels[side_corners[3][1]]) -
 						(corner_levels[side_corners[2][0]] +
 						corner_levels[side_corners[2][1]]);
 				// Positive if liquid moves towards +X
-				int dx = (corner_levels[side_corners[1][0]] +
+				f32 dx = (corner_levels[side_corners[1][0]] +
 						corner_levels[side_corners[1][1]]) -
 						(corner_levels[side_corners[0][0]] +
 						corner_levels[side_corners[0][1]]);
-				// -X
-				if(-dx >= abs(dz))
+				f32 tcoord_angle = atan2(dz, dx) * core::RADTODEG ;
+				v2f tcoord_center(0.5, 0.5);
+				v2f tcoord_translate(
+						blockpos_nodes.Z + z,
+						blockpos_nodes.X + x);
+				tcoord_translate.rotateBy(tcoord_angle);
+				tcoord_translate.X -= floor(tcoord_translate.X);
+				tcoord_translate.Y -= floor(tcoord_translate.Y);
+
+				for(s32 i=0; i<4; i++)
 				{
-					v2f t = vertices[0].TCoords;
-					vertices[0].TCoords = vertices[1].TCoords;
-					vertices[1].TCoords = vertices[2].TCoords;
-					vertices[2].TCoords = vertices[3].TCoords;
-					vertices[3].TCoords = t;
-				}
-				// +X
-				if(dx >= abs(dz))
-				{
-					v2f t = vertices[0].TCoords;
-					vertices[0].TCoords = vertices[3].TCoords;
-					vertices[3].TCoords = vertices[2].TCoords;
-					vertices[2].TCoords = vertices[1].TCoords;
-					vertices[1].TCoords = t;
-				}
-				// -Z
-				if(-dz >= abs(dx))
-				{
-					v2f t = vertices[0].TCoords;
-					vertices[0].TCoords = vertices[3].TCoords;
-					vertices[3].TCoords = vertices[2].TCoords;
-					vertices[2].TCoords = vertices[1].TCoords;
-					vertices[1].TCoords = t;
-					t = vertices[0].TCoords;
-					vertices[0].TCoords = vertices[3].TCoords;
-					vertices[3].TCoords = vertices[2].TCoords;
-					vertices[2].TCoords = vertices[1].TCoords;
-					vertices[1].TCoords = t;
+					vertices[i].TCoords.rotateBy(
+							tcoord_angle,
+							tcoord_center);
+					vertices[i].TCoords += tcoord_translate;
 				}
 
+				v2f t = vertices[0].TCoords;
+				vertices[0].TCoords = vertices[2].TCoords;
+				vertices[2].TCoords = t;
+
 				u16 indices[] = {0,1,2,2,3,0};
 				// Add to mesh collector
 				collector.append(tile_liquid, vertices, 4, indices, 6);
@@ -725,7 +695,6 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 		case NDT_GLASSLIKE:
 		{
 			TileSpec tile = getNodeTile(n, p, v3s16(0,0,0), data);
-			AtlasPointer ap = tile.texture;
 
 			u16 l = getInteriorLight(n, 1, data);
 			video::SColor c = MapBlock_LightColor(255, l, decode_light(f.light_source));
@@ -741,14 +710,10 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 
 				// The face at Z+
 				video::S3DVertex vertices[4] = {
-					video::S3DVertex(-BS/2,-BS/2,BS/2, 0,0,0, c,
-						ap.x0(), ap.y1()),
-					video::S3DVertex(BS/2,-BS/2,BS/2, 0,0,0, c,
-						ap.x1(), ap.y1()),
-					video::S3DVertex(BS/2,BS/2,BS/2, 0,0,0, c,
-						ap.x1(), ap.y0()),
-					video::S3DVertex(-BS/2,BS/2,BS/2, 0,0,0, c,
-						ap.x0(), ap.y0()),
+					video::S3DVertex(-BS/2,-BS/2,BS/2, 0,0,0, c, 0,1),
+					video::S3DVertex(BS/2,-BS/2,BS/2, 0,0,0, c, 1,1),
+					video::S3DVertex(BS/2,BS/2,BS/2, 0,0,0, c, 1,0),
+					video::S3DVertex(-BS/2,BS/2,BS/2, 0,0,0, c, 0,0),
 				};
 				
 				// Rotations in the g_6dirs format
@@ -910,7 +875,6 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 		{
 			TileSpec tile_leaves = getNodeTile(n, p,
 					v3s16(0,0,0), data);
-			AtlasPointer pa_leaves = tile_leaves.texture;
 
 			u16 l = getInteriorLight(n, 1, data);
 			video::SColor c = MapBlock_LightColor(255, l, decode_light(f.light_source));
@@ -945,22 +909,17 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			tile.material_flags &= ~MATERIAL_FLAG_BACKFACE_CULLING;
 			tile.material_flags |= MATERIAL_FLAG_CRACK_OVERLAY;
 
-			AtlasPointer ap = tile.texture;
-
 			u16 l = getInteriorLight(n, 1, data);
 			video::SColor c = MapBlock_LightColor(255, l, decode_light(f.light_source));
 
+			float s = BS/2*f.visual_scale;
 			// Wall at X+ of node
 			video::S3DVertex vertices[4] =
 			{
-				video::S3DVertex(-BS/2,-BS/2,0, 0,0,0, c,
-						ap.x0(), ap.y1()),
-				video::S3DVertex(BS/2,-BS/2,0, 0,0,0, c,
-						ap.x1(), ap.y1()),
-				video::S3DVertex(BS/2,BS/2,0, 0,0,0, c,
-						ap.x1(), ap.y0()),
-				video::S3DVertex(-BS/2,BS/2,0, 0,0,0, c,
-						ap.x0(), ap.y0()),
+				video::S3DVertex(-s,-s,0, 0,0,0, c, 0,1),
+				video::S3DVertex( s,-s,0, 0,0,0, c, 1,1),
+				video::S3DVertex( s, s,0, 0,0,0, c, 1,0),
+				video::S3DVertex(-s, s,0, 0,0,0, c, 0,0),
 			};
 
 			for(s32 i=0; i<4; i++)
@@ -990,23 +949,19 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			TileSpec tile = getNodeTileN(n, p, 0, data);
 			tile.material_flags &= ~MATERIAL_FLAG_BACKFACE_CULLING;
 			tile.material_flags |= MATERIAL_FLAG_CRACK_OVERLAY;
-			AtlasPointer ap = tile.texture;
 
 			u16 l = getInteriorLight(n, 0, data);
 			video::SColor c = MapBlock_LightColor(255, l, decode_light(f.light_source));
 				
 			float d = (float)BS/16;
+			float s = BS/2*f.visual_scale;
 			// Wall at X+ of node
 			video::S3DVertex vertices[4] =
 			{
-				video::S3DVertex(BS/2-d,BS/2,BS/2, 0,0,0, c,
-						ap.x0(), ap.y0()),
-				video::S3DVertex(BS/2-d,BS/2,-BS/2, 0,0,0, c,
-						ap.x1(), ap.y0()),
-				video::S3DVertex(BS/2-d,-BS/2,-BS/2, 0,0,0, c,
-						ap.x1(), ap.y1()),
-				video::S3DVertex(BS/2-d,-BS/2,BS/2, 0,0,0, c,
-						ap.x0(), ap.y1()),
+				video::S3DVertex(BS/2-d,  s,  s, 0,0,0, c, 0,0),
+				video::S3DVertex(BS/2-d,  s, -s, 0,0,0, c, 1,0),
+				video::S3DVertex(BS/2-d, -s, -s, 0,0,0, c, 1,1),
+				video::S3DVertex(BS/2-d, -s,  s, 0,0,0, c, 0,1),
 			};
 
 			v3s16 dir = n.getWallMountedDir(nodedef);
@@ -1037,25 +992,20 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 		{
 			TileSpec tile = getNodeTileN(n, p, 0, data);
 			tile.material_flags |= MATERIAL_FLAG_CRACK_OVERLAY;
-			AtlasPointer ap = tile.texture;
 			
 			u16 l = getInteriorLight(n, 1, data);
 			video::SColor c = MapBlock_LightColor(255, l, decode_light(f.light_source));
 
+			float s = BS/2*f.visual_scale;
+
 			for(u32 j=0; j<2; j++)
 			{
 				video::S3DVertex vertices[4] =
 				{
-					video::S3DVertex(-BS/2*f.visual_scale,-BS/2,0, 0,0,0, c,
-						ap.x0(), ap.y1()),
-					video::S3DVertex( BS/2*f.visual_scale,-BS/2,0, 0,0,0, c,
-						ap.x1(), ap.y1()),
-					video::S3DVertex( BS/2*f.visual_scale,
-						-BS/2 + f.visual_scale*BS,0, 0,0,0, c,
-						ap.x1(), ap.y0()),
-					video::S3DVertex(-BS/2*f.visual_scale,
-						-BS/2 + f.visual_scale*BS,0, 0,0,0, c,
-						ap.x0(), ap.y0()),
+					video::S3DVertex(-s,-BS/2,      0, 0,0,0, c, 0,1),
+					video::S3DVertex( s,-BS/2,      0, 0,0,0, c, 1,1),
+					video::S3DVertex( s,-BS/2 + s*2,0, 0,0,0, c, 1,0),
+					video::S3DVertex(-s,-BS/2 + s*2,0, 0,0,0, c, 0,0),
 				};
 
 				if(j == 0)
@@ -1088,10 +1038,13 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			
 			// A hack to put wood the right way around in the posts
 			ITextureSource *tsrc = data->m_gamedef->tsrc();
+			std::string texturestring_rot = tsrc->getTextureName(
+					tile.texture_id) + "^[transformR90";
 			TileSpec tile_rot = tile;
-			tile_rot.texture = tsrc->getTexture(tsrc->getTextureName(
-					tile.texture.id) + "^[transformR90");
-					
+			tile_rot.texture = tsrc->getTexture(
+					texturestring_rot,
+					&tile_rot.texture_id);
+			
 			u16 l = getInteriorLight(n, 1, data);
 			video::SColor c = MapBlock_LightColor(255, l, decode_light(f.light_source));
 
@@ -1341,8 +1294,6 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 			tile.material_flags &= ~MATERIAL_FLAG_BACKFACE_CULLING;
 			tile.material_flags |= MATERIAL_FLAG_CRACK_OVERLAY;
 
-			AtlasPointer ap = tile.texture;
-			
 			u16 l = getInteriorLight(n, 0, data);
 			video::SColor c = MapBlock_LightColor(255, l, decode_light(f.light_source));
 
@@ -1354,14 +1305,10 @@ void mapblock_mesh_generate_special(MeshMakeData *data,
 
 			video::S3DVertex vertices[4] =
 			{
-					video::S3DVertex(-BS/2,-BS/2+d,-BS/2, 0,0,0, c,
-							ap.x0(), ap.y1()),
-					video::S3DVertex(BS/2,-BS/2+d,-BS/2, 0,0,0, c,
-							ap.x1(), ap.y1()),
-					video::S3DVertex(BS/2,g*BS/2+d,BS/2, 0,0,0, c,
-							ap.x1(), ap.y0()),
-					video::S3DVertex(-BS/2,g*BS/2+d,BS/2, 0,0,0, c,
-							ap.x0(), ap.y0()),
+					video::S3DVertex(-BS/2,-BS/2+d,-BS/2, 0,0,0, c, 0,1),
+					video::S3DVertex(BS/2,-BS/2+d,-BS/2, 0,0,0, c, 1,1),
+					video::S3DVertex(BS/2,g*BS/2+d,BS/2, 0,0,0, c, 1,0),
+					video::S3DVertex(-BS/2,g*BS/2+d,BS/2, 0,0,0, c, 0,0),
 			};
 
 			for(s32 i=0; i<4; i++)
