@@ -21,8 +21,14 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define EMERGE_HEADER
 
 #include <map>
-#include <queue>
-#include "util/thread.h"
+#include "irr_v3d.h"
+#include "util/container.h"
+#include "map.h" // for ManualMapVoxelManipulator
+
+#define MGPARAMS_SET_MGNAME      1
+#define MGPARAMS_SET_SEED        2
+#define MGPARAMS_SET_WATER_LEVEL 4
+#define MGPARAMS_SET_FLAGS       8
 
 #define BLOCK_EMERGE_ALLOWGEN (1<<0)
 
@@ -30,15 +36,16 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 	{ if (enable_mapgen_debug_info) \
 	infostream << "EmergeThread: " x << std::endl; }
 
+class EmergeThread;
 class Mapgen;
 struct MapgenParams;
 struct MapgenFactory;
 class Biome;
 class BiomeDefManager;
-class EmergeThread;
-class ManualMapVoxelManipulator;
-
-#include "server.h"
+class Decoration;
+class Ore;
+class INodeDefManager;
+class Settings;
 
 struct BlockMakeData {
 	ManualMapVoxelManipulator *vmanip;
@@ -63,22 +70,36 @@ struct BlockEmergeData {
 	u8 flags;
 };
 
-class EmergeManager {
+class IBackgroundBlockEmerger
+{
+public:
+	virtual bool enqueueBlockEmerge(u16 peer_id, v3s16 p,
+			bool allow_generate) = 0;
+	virtual ~IBackgroundBlockEmerger() {}
+};
+
+class EmergeManager : public IBackgroundBlockEmerger {
 public:
 	INodeDefManager *ndef;
 
 	std::map<std::string, MapgenFactory *> mglist;
-	
+
 	std::vector<Mapgen *> mapgen;
 	std::vector<EmergeThread *> emergethread;
-	
+
 	//settings
 	MapgenParams *params;
 	bool mapgen_debug_info;
 	u16 qlimit_total;
 	u16 qlimit_diskonly;
 	u16 qlimit_generate;
-	
+
+	u32 gennotify;
+
+	MapgenParams *luaoverride_params;
+	u32 luaoverride_params_modified;
+	u32 luaoverride_flagmask;
+
 	//block emerge queue data structures
 	JMutex queuemutex;
 	std::map<v3s16, BlockEmergeData *> blocks_enqueued;
@@ -87,20 +108,24 @@ public:
 	//Mapgen-related structures
 	BiomeDefManager *biomedef;
 	std::vector<Ore *> ores;
+	std::vector<Decoration *> decorations;
 
 	EmergeManager(IGameDef *gamedef);
 	~EmergeManager();
 
 	void initMapgens(MapgenParams *mgparams);
+	MapgenParams *setMapgenType(MapgenParams *mgparams, std::string newname);
+	Mapgen *getCurrentMapgen();
 	Mapgen *createMapgen(std::string mgname, int mgid,
 						MapgenParams *mgparams);
 	MapgenParams *createMapgenParams(std::string mgname);
+	void startAllThreads();
 	bool enqueueBlockEmerge(u16 peer_id, v3s16 p, bool allow_generate);
-	
+
 	void registerMapgen(std::string name, MapgenFactory *mgfactory);
 	MapgenParams *getParamsFromSettings(Settings *settings);
 	void setParamsToSettings(Settings *settings);
-	
+
 	//mapgen helper methods
 	Biome *getBiomeAtPoint(v3s16 p);
 	int getGroundLevelAtPoint(v2s16 p);
@@ -108,43 +133,4 @@ public:
 	u32 getBlockSeed(v3s16 p);
 };
 
-class EmergeThread : public SimpleThread
-{
-	Server *m_server;
-	ServerMap *map;
-	EmergeManager *emerge;
-	Mapgen *mapgen;
-	bool enable_mapgen_debug_info;
-	int id;
-	
-public:
-	Event qevent;
-	std::queue<v3s16> blockqueue;
-	
-	EmergeThread(Server *server, int ethreadid):
-		SimpleThread(),
-		m_server(server),
-		map(NULL),
-		emerge(NULL),
-		mapgen(NULL),
-		id(ethreadid)
-	{
-	}
-
-	void *Thread();
-
-	void trigger()
-	{
-		setRun(true);
-		if(IsRunning() == false)
-		{
-			Start();
-		}
-	}
-
-	bool popBlockEmerge(v3s16 *pos, u8 *flags);
-	bool getBlockOrStartGen(v3s16 p, MapBlock **b, 
-							BlockMakeData *data, bool allow_generate);
-};
-
 #endif
