@@ -32,11 +32,9 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #include <set>
 #include <list>
-#include "irrlichttypes_extrabloated.h"
-#include "player.h"
-#include <ostream>
+#include <map>
+#include "irr_v3d.h"
 #include "activeobject.h"
-#include "util/container.h"
 #include "util/numeric.h"
 #include "mapnode.h"
 #include "mapblock.h"
@@ -44,13 +42,14 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 class ServerEnvironment;
 class ActiveBlockModifier;
 class ServerActiveObject;
-typedef struct lua_State lua_State;
 class ITextureSource;
 class IGameDef;
+class IBackgroundBlockEmerger;
 class Map;
 class ServerMap;
 class ClientMap;
-class ScriptApi;
+class GameScripting;
+class Player;
 
 class Environment
 {
@@ -77,7 +76,6 @@ public:
 	Player * getNearestConnectedPlayer(v3f pos);
 	std::list<Player*> getPlayers();
 	std::list<Player*> getPlayers(bool ignore_disconnected);
-	void printPlayers(std::ostream &o);
 	
 	u32 getDayNightRatio();
 	
@@ -176,12 +174,6 @@ public:
 private:
 };
 
-class IBackgroundBlockEmerger
-{
-public:
-	virtual void queueBlockEmerge(v3s16 blockpos, bool allow_generate)=0;
-};
-
 /*
 	The server-side environment.
 
@@ -191,7 +183,8 @@ public:
 class ServerEnvironment : public Environment
 {
 public:
-	ServerEnvironment(ServerMap *map, ScriptApi *iface, IGameDef *gamedef,
+	ServerEnvironment(ServerMap *map, GameScripting *scriptIface,
+			IGameDef *gamedef,
 			IBackgroundBlockEmerger *emerger);
 	~ServerEnvironment();
 
@@ -200,7 +193,7 @@ public:
 	ServerMap & getServerMap();
 
 	//TODO find way to remove this fct!
-	ScriptApi* getScriptIface()
+	GameScripting* getScriptIface()
 		{ return m_script; }
 
 	IGameDef *getGameDef()
@@ -290,6 +283,7 @@ public:
 	// Script-aware node setters
 	bool setNode(v3s16 p, const MapNode &n);
 	bool removeNode(v3s16 p);
+	bool swapNode(v3s16 p, const MapNode &n);
 	
 	// Find all active objects inside a radius around a point
 	std::set<u16> getObjectsInsideRadius(v3f pos, float radius);
@@ -301,8 +295,16 @@ public:
 	void step(f32 dtime);
 	
 	//check if there's a line of sight between two positions
-	bool line_of_sight(v3f pos1, v3f pos2, float stepsize=1.0);
+	bool line_of_sight(v3f pos1, v3f pos2, float stepsize=1.0, v3s16 *p=NULL);
 
+	u32 getGameTime() { return m_game_time; }
+
+	void reportMaxLagEstimate(float f) { m_max_lag_estimate = f; }
+	float getMaxLagEstimate() { return m_max_lag_estimate; }
+	
+	// is weather active in this environment?
+	bool m_use_weather;
+	
 private:
 
 	/*
@@ -350,15 +352,15 @@ private:
 	// The map
 	ServerMap *m_map;
 	// Lua state
-	ScriptApi* m_script;
+	GameScripting* m_script;
 	// Game definition
 	IGameDef *m_gamedef;
-	// Background block emerger (the server, in practice)
+	// Background block emerger (the EmergeManager, in practice)
 	IBackgroundBlockEmerger *m_emerger;
 	// Active object list
 	std::map<u16, ServerActiveObject*> m_active_objects;
 	// Outgoing network message buffer for active objects
-	Queue<ActiveObjectMessage> m_active_object_messages;
+	std::list<ActiveObjectMessage> m_active_object_messages;
 	// Some timers
 	float m_random_spawn_timer; // used for experimental code
 	float m_send_recommended_timer;
@@ -377,6 +379,9 @@ private:
 	std::list<ABMWithState> m_abms;
 	// An interval for generally sending object positions and stuff
 	float m_recommended_send_interval;
+	// Estimate for general maximum lag as determined by server.
+	// Can raise to high values like 15s with eg. map generation mods.
+	float m_max_lag_estimate;
 };
 
 #ifndef SERVER
@@ -395,7 +400,8 @@ class ClientSimpleObject;
 enum ClientEnvEventType
 {
 	CEE_NONE,
-	CEE_PLAYER_DAMAGE
+	CEE_PLAYER_DAMAGE,
+	CEE_PLAYER_BREATH
 };
 
 struct ClientEnvEvent
@@ -408,6 +414,9 @@ struct ClientEnvEvent
 			u8 amount;
 			bool send_to_server;
 		} player_damage;
+		struct{
+			u16 amount;
+		} player_breath;
 	};
 };
 
@@ -462,6 +471,7 @@ public:
 	*/
 
 	void damageLocalPlayer(u8 damage, bool handle_hp=true);
+	void updateLocalPlayerBreath(u16 breath);
 
 	/*
 		Client likes to call these
@@ -491,9 +501,11 @@ private:
 	IrrlichtDevice *m_irr;
 	std::map<u16, ClientActiveObject*> m_active_objects;
 	std::list<ClientSimpleObject*> m_simple_objects;
-	Queue<ClientEnvEvent> m_client_event_queue;
+	std::list<ClientEnvEvent> m_client_event_queue;
 	IntervalLimiter m_active_object_light_update_interval;
 	IntervalLimiter m_lava_hurt_interval;
+	IntervalLimiter m_drowning_interval;
+	IntervalLimiter m_breathing_interval;
 	std::list<std::string> m_player_names;
 };
 
