@@ -100,7 +100,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 		Add stuff
 	*/
 	s32 ypos = 50;
-	changeCtype("");
 	{
 		core::rect<s32> rect(0, 0, 110, 20);
 		rect += topleft_client + v2s32(35, ypos+6);
@@ -108,7 +107,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 		Environment->addStaticText(text, rect, false, true, this, -1);
 		delete[] text;
 	}
-	changeCtype("C");
 	{
 		core::rect<s32> rect(0, 0, 230, 30);
 		rect += topleft_client + v2s32(160, ypos);
@@ -118,7 +116,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 		e->setPasswordBox(true);
 	}
 	ypos += 50;
-	changeCtype("");
 	{
 		core::rect<s32> rect(0, 0, 110, 20);
 		rect += topleft_client + v2s32(35, ypos+6);
@@ -126,7 +123,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 		Environment->addStaticText(text, rect, false, true, this, -1);
 		delete[] text;
 	}
-	changeCtype("C");
 	{
 		core::rect<s32> rect(0, 0, 230, 30);
 		rect += topleft_client + v2s32(160, ypos);
@@ -135,7 +131,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 		e->setPasswordBox(true);
 	}
 	ypos += 50;
-	changeCtype("");
 	{
 		core::rect<s32> rect(0, 0, 110, 20);
 		rect += topleft_client + v2s32(35, ypos+6);
@@ -143,7 +138,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 		Environment->addStaticText(text, rect, false, true, this, -1);
 		delete[] text;
 	}
-	changeCtype("C");
 	{
 		core::rect<s32> rect(0, 0, 230, 30);
 		rect += topleft_client + v2s32(160, ypos);
@@ -153,7 +147,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 	}
 
 	ypos += 50;
-	changeCtype("");
 	{
 		core::rect<s32> rect(0, 0, 140, 30);
 		rect = rect + v2s32(size.X/2-140/2, ypos);
@@ -174,8 +167,6 @@ void GUIPasswordChange::regenerateGui(v2u32 screensize)
 		e->setVisible(false);
 		delete[] text;
 	}
-	changeCtype("C");
-
 }
 
 void GUIPasswordChange::drawMenu()
