@@ -26,9 +26,6 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "irr_v3d.h"
 #include "irr_aabb3d.h"
 
-#include <irrMap.h>
-#include <irrList.h>
-#include <irrArray.h>
 #include <SColor.h>
 
 #endif
