@@ -61,6 +61,9 @@ public:
 
 	float camera_impact;
 
+	std::string hotbar_image;
+	std::string hotbar_selected_image;
+
 private:
 	// This is used for determining the sneaking range
 	v3s16 m_sneak_node;
