@@ -20,9 +20,6 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #ifndef MAP_HEADER
 #define MAP_HEADER
 
-#include <jmutex.h>
-#include <jmutexautolock.h>
-#include <jthread.h>
 #include <iostream>
 #include <sstream>
 #include <set>
@@ -33,15 +30,11 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "mapnode.h"
 #include "constants.h"
 #include "voxel.h"
-#include "mapgen.h" //for BlockMakeData and EmergeManager
 #include "modifiedstate.h"
 #include "util/container.h"
 #include "nodetimer.h"
 
-extern "C" {
-	#include "sqlite3.h"
-}
-
+class Database;
 class ClientMap;
 class MapSector;
 class ServerMapSector;
@@ -50,7 +43,9 @@ class NodeMetadata;
 class IGameDef;
 class IRollbackReportSink;
 class EmergeManager;
+class ServerEnvironment;
 struct BlockMakeData;
+struct MapgenParams;
 
 
 /*
@@ -66,6 +61,8 @@ enum MapEditEventType{
 	MEET_ADDNODE,
 	// Node removed (changed to air)
 	MEET_REMOVENODE,
+	// Node swapped (changed without metadata change)
+	MEET_SWAPNODE,
 	// Node metadata of block changed (not knowing which node exactly)
 	// p stores block coordinate
 	MEET_BLOCK_NODE_METADATA_CHANGED,
@@ -104,6 +101,8 @@ struct MapEditEvent
 			return VoxelArea(p);
 		case MEET_REMOVENODE:
 			return VoxelArea(p);
+		case MEET_SWAPNODE:
+			return VoxelArea(p);
 		case MEET_BLOCK_NODE_METADATA_CHANGED:
 		{
 			v3s16 np1 = p*MAP_BLOCKSIZE;
@@ -241,7 +240,8 @@ public:
 		These handle lighting but not faces.
 	*/
 	void addNodeAndUpdate(v3s16 p, MapNode n,
-			std::map<v3s16, MapBlock*> &modified_blocks);
+			std::map<v3s16, MapBlock*> &modified_blocks,
+			bool remove_metadata = true);
 	void removeNodeAndUpdate(v3s16 p,
 			std::map<v3s16, MapBlock*> &modified_blocks);
 
@@ -250,7 +250,7 @@ public:
 		These emit events.
 		Return true if succeeded, false if not.
 	*/
-	bool addNodeWithEvent(v3s16 p, MapNode n);
+	bool addNodeWithEvent(v3s16 p, MapNode n, bool remove_metadata = true);
 	bool removeNodeWithEvent(v3s16 p);
 
 	/*
@@ -312,7 +312,22 @@ public:
 	*/
 
 	NodeMetadata* getNodeMetadata(v3s16 p);
-	void setNodeMetadata(v3s16 p, NodeMetadata *meta);
+
+	/**
+	 * Sets metadata for a node.
+	 * This method sets the metadata for a given node.
+	 * On success, it returns @c true and the object pointed to
+	 * by @p meta is then managed by the system and should
+	 * not be deleted by the caller.
+	 *
+	 * In case of failure, the method returns @c false and the
+	 * caller is still responsible for deleting the object!
+	 *
+	 * @param p node coordinates
+	 * @param meta pointer to @c NodeMetadata object
+	 * @return @c true on success, false on failure
+	 */
+	bool setNodeMetadata(v3s16 p, NodeMetadata *meta);
 	void removeNodeMetadata(v3s16 p);
 
 	/*
@@ -336,7 +351,11 @@ public:
 	void transforming_liquid_add(v3s16 p);
 	s32 transforming_liquid_size();
 
+	virtual s16 getHeat(v3s16 p);
+	virtual s16 getHumidity(v3s16 p);
+
 protected:
+	friend class LuaVoxelManip;
 
 	std::ostream &m_dout; // A bit deprecated, could be removed
 
@@ -404,9 +423,12 @@ public:
 
 	*/
 	MapBlock * emergeBlock(v3s16 p, bool create_blank=true);
+	
+	// Carries out any initialization necessary before block is sent
+	void prepareBlock(MapBlock *block);
 
 	// Helper for placing objects on ground level
-	s16 findGroundLevel(v2s16 p2d);
+	s16 findGroundLevel(v2s16 p2d, bool cacheBlocks);
 
 	/*
 		Misc. helper functions for fiddling with directory and file
@@ -423,13 +445,8 @@ public:
 	/*
 		Database functions
 	*/
-	// Create the database structure
-	void createDatabase();
 	// Verify we can read/write to the database
 	void verifyDatabase();
-	// Get an integer suitable for a block
-	static sqlite3_int64 getBlockAsInteger(const v3s16 pos);
-	static v3s16 getIntegerAsBlock(sqlite3_int64 i);
 
 	// Returns true if the database file does not exist
 	bool loadFromFolders();
@@ -479,9 +496,14 @@ public:
 	u64 getSeed(){ return m_seed; }
 
 	MapgenParams *getMapgenParams(){ return m_mgparams; }
+	void setMapgenParams(MapgenParams *mgparams){ m_mgparams = mgparams; }
 
 	// Parameters fed to the Mapgen
 	MapgenParams *m_mgparams;
+
+	virtual s16 updateBlockHeat(ServerEnvironment *env, v3s16 p, MapBlock *block = NULL);
+	virtual s16 updateBlockHumidity(ServerEnvironment *env, v3s16 p, MapBlock *block = NULL);
+
 private:
 	// Seed used for all kinds of randomness in generation
 	u64 m_seed;
@@ -505,14 +527,7 @@ private:
 		This is reset to false when written on disk.
 	*/
 	bool m_map_metadata_changed;
-
-	/*
-		SQLite database and statements
-	*/
-	sqlite3 *m_database;
-	sqlite3_stmt *m_database_read;
-	sqlite3_stmt *m_database_write;
-	sqlite3_stmt *m_database_list;
+	Database *dbase;
 };
 
 #define VMANIP_BLOCK_DATA_INEXIST     1
@@ -554,7 +569,8 @@ public:
 
 	virtual void emerge(VoxelArea a, s32 caller_id=-1);
 
-	void initialEmerge(v3s16 blockpos_min, v3s16 blockpos_max);
+	void initialEmerge(v3s16 blockpos_min, v3s16 blockpos_max,
+						bool load_if_inexistent = true);
 
 	// This is much faster with big chunks of generated data
 	void blitBackAll(std::map<v3s16, MapBlock*> * modified_blocks);
