@@ -84,7 +84,6 @@ public:
 	EmergeManager *emerge;
 
 	int ystride;
-	v3s16 csize;
 	u32 flags;
 
 	u32 blockseed;
@@ -108,7 +107,7 @@ public:
 	NoiseParams *np_apple_trees;
 	float freq_desert;
 	float freq_beach;
-	
+
 	content_t c_stone;
 	content_t c_dirt;
 	content_t c_dirt_with_grass;
@@ -120,6 +119,11 @@ public:
 	content_t c_desert_sand;
 	content_t c_desert_stone;
 
+	content_t c_mossycobble;
+	content_t c_sandbrick;
+	content_t c_stair_cobble;
+	content_t c_stair_sandstone;
+
 	MapgenV6(int mapgenid, MapgenV6Params *params, EmergeManager *emerge);
 	~MapgenV6();
 	
@@ -132,7 +136,6 @@ public:
 	virtual float baseTerrainLevelFromMap(v2s16 p);
 	virtual float baseTerrainLevelFromMap(int index);
 
-	s16 find_ground_level(v2s16 p2d);
 	s16 find_stone_level(v2s16 p2d);
 	bool block_is_underground(u64 seed, v3s16 blockpos);
 	s16 find_ground_level_from_noise(u64 seed, v2s16 p2d, s16 precision);
