@@ -160,6 +160,16 @@ public:
 		return m_yaw;
 	}
 
+	u16 getBreath()
+	{
+		return m_breath;
+	}
+
+	virtual void setBreath(u16 breath)
+	{
+		m_breath = breath;
+	}
+
 	f32 getRadPitch()
 	{
 		return -1.0 * m_pitch * core::DEGTORAD;
@@ -184,6 +194,15 @@ public:
 		return m_collisionbox;
 	}
 
+	u32 getFreeHudID() const {
+		size_t size = hud.size();
+		for (size_t i = 0; i != size; i++) {
+			if (!hud[i])
+				return i;
+		}
+		return size;
+	}
+
 	virtual bool isLocal() const
 	{ return false; }
 	virtual PlayerSAO *getPlayerSAO()
@@ -197,7 +216,24 @@ public:
 		deSerialize stops reading exactly at the right point.
 	*/
 	void serialize(std::ostream &os);
-	void deSerialize(std::istream &is);
+	void deSerialize(std::istream &is, std::string playername);
+
+	bool checkModified()
+	{
+		if(m_last_hp != hp || m_last_pitch != m_pitch ||
+				m_last_pos != m_position || m_last_yaw != m_yaw ||
+				!(inventory == m_last_inventory))
+		{
+			m_last_hp = hp;
+			m_last_pitch = m_pitch;
+			m_last_pos = m_position;
+			m_last_yaw = m_yaw;
+			m_last_inventory = inventory;
+			return true;
+		} else {
+			return false;
+		}
+	}
 
 	bool touching_ground;
 	// This oscillates so that the player jumps a bit above the surface
@@ -230,6 +266,8 @@ public:
 	float physics_override_speed;
 	float physics_override_jump;
 	float physics_override_gravity;
+	bool physics_override_sneak;
+	bool physics_override_sneak_glitch;
 
 	u16 hp;
 
@@ -237,7 +275,7 @@ public:
 	float hurt_tilt_strength;
 
 	u16 peer_id;
-	
+
 	std::string inventory_formspec;
 	
 	PlayerControl control;
@@ -256,11 +294,18 @@ protected:
 	IGameDef *m_gamedef;
 
 	char m_name[PLAYERNAME_SIZE];
+	u16 m_breath;
 	f32 m_pitch;
 	f32 m_yaw;
 	v3f m_speed;
 	v3f m_position;
 	core::aabbox3d<f32> m_collisionbox;
+
+	f32 m_last_pitch;
+	f32 m_last_yaw;
+	v3f m_last_pos;
+	u16 m_last_hp;
+	Inventory m_last_inventory;
 };
 
 
