@@ -29,7 +29,9 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "server.h"
 #include "log.h"
 #include "tool.h"
-#include "server.h"
+#include "serverobject.h"
+#include "mapgen.h"
+#include "json/json.h"
 
 struct EnumString es_TileAnimationType[] =
 {
@@ -99,6 +101,8 @@ ItemDefinition read_item_definition(lua_State* L,int index,
 	}
 	lua_pop(L, 1);
 
+	def.range = getfloatfield_default(L, index, "range", def.range);
+
 	// Client shall immediately place this node when player places the item.
 	// Server will update the precise end result a moment later.
 	// "" = no prediction
@@ -120,6 +124,7 @@ void read_object_properties(lua_State *L, int index,
 	prop->hp_max = getintfield_default(L, -1, "hp_max", 10);
 
 	getboolfield(L, -1, "physical", prop->physical);
+	getboolfield(L, -1, "collide_with_objects", prop->collideWithObjects);
 
 	getfloatfield(L, -1, "weight", prop->weight);
 
@@ -184,6 +189,17 @@ void read_object_properties(lua_State *L, int index,
 	getboolfield(L, -1, "is_visible", prop->is_visible);
 	getboolfield(L, -1, "makes_footstep_sound", prop->makes_footstep_sound);
 	getfloatfield(L, -1, "automatic_rotate", prop->automatic_rotate);
+	getfloatfield(L, -1, "stepheight", prop->stepheight);
+	prop->stepheight*=BS;
+	lua_getfield(L, -1, "automatic_face_movement_dir");
+	if (lua_isnumber(L, -1)) {
+		prop->automatic_face_movement_dir = true;
+		prop->automatic_face_movement_dir_offset = luaL_checknumber(L, -1);
+	} else if (lua_isboolean(L, -1)) {
+		prop->automatic_face_movement_dir = lua_toboolean(L, -1);
+		prop->automatic_face_movement_dir_offset = 0.0;
+	}
+	lua_pop(L, 1);
 }
 
 /******************************************************************************/
@@ -318,7 +334,7 @@ ContentFeatures read_content_features(lua_State *L, int index)
 			// removes value, keeps key for next iteration
 			lua_pop(L, 1);
 			i++;
-			if(i==6){
+			if(i==CF_SPECIAL_COUNT){
 				lua_pop(L, 1);
 				break;
 			}
@@ -388,7 +404,14 @@ ContentFeatures read_content_features(lua_State *L, int index)
 	// the slowest possible
 	f.liquid_viscosity = getintfield_default(L, index,
 			"liquid_viscosity", f.liquid_viscosity);
+	f.liquid_range = getintfield_default(L, index,
+			"liquid_range", f.liquid_range);
+	f.leveled = getintfield_default(L, index, "leveled", f.leveled);
+
 	getboolfield(L, index, "liquid_renewable", f.liquid_renewable);
+	getstringfield(L, index, "freezemelt", f.freezemelt);
+	f.drowning = getintfield_default(L, index,
+			"drowning", f.drowning);
 	// Amount of light the node emits
 	f.light_source = getintfield_default(L, index,
 			"light_source", f.light_source);
@@ -405,6 +428,9 @@ ContentFeatures read_content_features(lua_State *L, int index)
 		f.selection_box = read_nodebox(L, -1);
  	lua_pop(L, 1);
 
+	f.waving = getintfield_default(L, index,
+			"waving", f.waving);
+
 	// Set to true if paramtype used to be 'facedir_simple'
 	getboolfield(L, index, "legacy_facedir_simple", f.legacy_facedir_simple);
 	// Set to true if wall_mounted used to be set to true
@@ -627,7 +653,7 @@ ItemStack read_item(lua_State* L, int index,Server* srv)
 	}
 	else
 	{
-		throw LuaError(L, "Expecting itemstack, itemstring, table or nil");
+		throw LuaError(NULL, "Expecting itemstack, itemstring, table or nil");
 	}
 }
 
@@ -680,8 +706,7 @@ void push_tool_capabilities(lua_State *L,
 }
 
 /******************************************************************************/
-void push_inventory_list(Inventory *inv, const char *name,
-		lua_State *L)
+void push_inventory_list(lua_State *L, Inventory *inv, const char *name)
 {
 	InventoryList *invlist = inv->getList(name);
 	if(invlist == NULL){
@@ -695,8 +720,8 @@ void push_inventory_list(Inventory *inv, const char *name,
 }
 
 /******************************************************************************/
-void read_inventory_list(Inventory *inv, const char *name,
-		lua_State *L, int tableindex, Server* srv,int forcesize)
+void read_inventory_list(lua_State *L, int tableindex,
+		Inventory *inv, const char *name, Server* srv, int forcesize)
 {
 	if(tableindex < 0)
 		tableindex = lua_gettop(L) + 1 + tableindex;
@@ -849,27 +874,17 @@ void read_groups(lua_State *L, int index,
 /******************************************************************************/
 void push_items(lua_State *L, const std::vector<ItemStack> &items)
 {
-	// Get the table insert function
-	lua_getglobal(L, "table");
-	lua_getfield(L, -1, "insert");
-	int table_insert = lua_gettop(L);
 	// Create and fill table
-	lua_newtable(L);
-	int table = lua_gettop(L);
-	for(u32 i=0; i<items.size(); i++){
-		ItemStack item = items[i];
-		lua_pushvalue(L, table_insert);
-		lua_pushvalue(L, table);
-		LuaItemStack::create(L, item);
-		if(lua_pcall(L, 2, 0, 0))
-			script_error(L, "error: %s", lua_tostring(L, -1));
+	lua_createtable(L, items.size(), 0);
+	std::vector<ItemStack>::const_iterator iter = items.begin();
+	for (u32 i = 0; iter != items.end(); iter++) {
+		LuaItemStack::create(L, *iter);
+		lua_rawseti(L, -2, ++i);
 	}
-	lua_remove(L, -2); // Remove table
-	lua_remove(L, -2); // Remove insert
 }
 
 /******************************************************************************/
-std::vector<ItemStack> read_items(lua_State *L, int index,Server* srv)
+std::vector<ItemStack> read_items(lua_State *L, int index, Server *srv)
 {
 	if(index < 0)
 		index = lua_gettop(L) + 1 + index;
@@ -877,10 +892,15 @@ std::vector<ItemStack> read_items(lua_State *L, int index,Server* srv)
 	std::vector<ItemStack> items;
 	luaL_checktype(L, index, LUA_TTABLE);
 	lua_pushnil(L);
-	while(lua_next(L, index) != 0){
-		// key at index -2 and value at index -1
-		items.push_back(read_item(L, -1, srv));
-		// removes value, keeps key for next iteration
+	while (lua_next(L, index)) {
+		s32 key = luaL_checkinteger(L, -2);
+		if (key < 1) {
+			throw LuaError(NULL, "Invalid inventory list index");
+		}
+		if (items.size() < (u32) key) {
+			items.resize(key);
+		}
+		items[key - 1] = read_item(L, -1, srv);
 		lua_pop(L, 1);
 	}
 	return items;
@@ -921,3 +941,197 @@ NoiseParams *read_noiseparams(lua_State *L, int index)
 
 	return np;
 }
+
+/******************************************************************************/
+bool read_schematic(lua_State *L, int index, DecoSchematic *dschem, Server *server) {
+	if (index < 0)
+		index = lua_gettop(L) + 1 + index;
+
+	INodeDefManager *ndef = server->getNodeDefManager();
+
+	if (lua_istable(L, index)) {
+		lua_getfield(L, index, "size");
+		v3s16 size = read_v3s16(L, -1);
+		lua_pop(L, 1);
+		
+		int numnodes = size.X * size.Y * size.Z;
+		MapNode *schemdata = new MapNode[numnodes];
+		int i = 0;
+		
+		lua_getfield(L, index, "data");
+		luaL_checktype(L, -1, LUA_TTABLE);
+		
+		lua_pushnil(L);
+		while (lua_next(L, -2)) {
+			if (i < numnodes) {
+				// same as readnode, except param1 default is MTSCHEM_PROB_CONST
+				lua_getfield(L, -1, "name");
+				const char *name = luaL_checkstring(L, -1);
+				lua_pop(L, 1);
+				
+				u8 param1;
+				lua_getfield(L, -1, "param1");
+				param1 = !lua_isnil(L, -1) ? lua_tonumber(L, -1) : MTSCHEM_PROB_ALWAYS;
+				lua_pop(L, 1);
+	
+				u8 param2;
+				lua_getfield(L, -1, "param2");
+				param2 = !lua_isnil(L, -1) ? lua_tonumber(L, -1) : 0;
+				lua_pop(L, 1);
+				
+				schemdata[i] = MapNode(ndef, name, param1, param2);
+			}
+			
+			i++;
+			lua_pop(L, 1);
+		}
+		
+		dschem->size      = size;
+		dschem->schematic = schemdata;
+		
+		if (i != numnodes) {
+			errorstream << "read_schematic: incorrect number of "
+				"nodes provided in raw schematic data (got " << i <<
+				", expected " << numnodes << ")." << std::endl;
+			return false;
+		}
+	} else if (lua_isstring(L, index)) {
+		dschem->filename = std::string(lua_tostring(L, index));
+	} else {
+		errorstream << "read_schematic: missing schematic "
+			"filename or raw schematic data" << std::endl;
+		return false;
+	}
+	
+	return true;
+}
+
+/******************************************************************************/
+// Returns depth of json value tree
+static int push_json_value_getdepth(const Json::Value &value)
+{
+	if (!value.isArray() && !value.isObject())
+		return 1;
+
+	int maxdepth = 0;
+	for (Json::Value::const_iterator it = value.begin();
+			it != value.end(); ++it) {
+		int elemdepth = push_json_value_getdepth(*it);
+		if (elemdepth > maxdepth)
+			maxdepth = elemdepth;
+	}
+	return maxdepth + 1;
+}
+// Recursive function to convert JSON --> Lua table
+static bool push_json_value_helper(lua_State *L, const Json::Value &value,
+		int nullindex)
+{
+	switch(value.type()) {
+		case Json::nullValue:
+		default:
+			lua_pushvalue(L, nullindex);
+			break;
+		case Json::intValue:
+			lua_pushinteger(L, value.asInt());
+			break;
+		case Json::uintValue:
+			lua_pushinteger(L, value.asUInt());
+			break;
+		case Json::realValue:
+			lua_pushnumber(L, value.asDouble());
+			break;
+		case Json::stringValue:
+			{
+				const char *str = value.asCString();
+				lua_pushstring(L, str ? str : "");
+			}
+			break;
+		case Json::booleanValue:
+			lua_pushboolean(L, value.asInt());
+			break;
+		case Json::arrayValue:
+			lua_newtable(L);
+			for (Json::Value::const_iterator it = value.begin();
+					it != value.end(); ++it) {
+				push_json_value_helper(L, *it, nullindex);
+				lua_rawseti(L, -2, it.index() + 1);
+			}
+			break;
+		case Json::objectValue:
+			lua_newtable(L);
+			for (Json::Value::const_iterator it = value.begin();
+					it != value.end(); ++it) {
+				const char *str = it.memberName();
+				lua_pushstring(L, str ? str : "");
+				push_json_value_helper(L, *it, nullindex);
+				lua_rawset(L, -3);
+			}
+			break;
+	}
+	return true;
+}
+// converts JSON --> Lua table; returns false if lua stack limit exceeded
+// nullindex: Lua stack index of value to use in place of JSON null
+bool push_json_value(lua_State *L, const Json::Value &value, int nullindex)
+{
+	if(nullindex < 0)
+		nullindex = lua_gettop(L) + 1 + nullindex;
+
+	int depth = push_json_value_getdepth(value);
+
+	// The maximum number of Lua stack slots used at each recursion level
+	// of push_json_value_helper is 2, so make sure there a depth * 2 slots
+	if (lua_checkstack(L, depth * 2))
+		return push_json_value_helper(L, value, nullindex);
+	else
+		return false;
+}
+
+// Converts Lua table --> JSON
+void get_json_value(lua_State *L, Json::Value &root, int index)
+{
+	int type = lua_type(L, index);
+	if (type == LUA_TBOOLEAN) {
+		root = (bool) lua_toboolean(L, index);
+	} else if (type == LUA_TNUMBER) {
+		root = lua_tonumber(L, index);
+	} else if (type == LUA_TSTRING) {
+		size_t len;
+		const char *str = lua_tolstring(L, index, &len);
+		root = std::string(str, len);
+	} else if (type == LUA_TTABLE) {
+		lua_pushnil(L);
+		while (lua_next(L, index)) {
+			// Key is at -2 and value is at -1
+			Json::Value value;
+			get_json_value(L, value, lua_gettop(L));
+
+			Json::ValueType roottype = root.type();
+			int keytype = lua_type(L, -1);
+			if (keytype == LUA_TNUMBER) {
+				lua_Number key = lua_tonumber(L, -1);
+				if (roottype != Json::nullValue && roottype != Json::arrayValue) {
+					throw SerializationError("Can't mix array and object values in JSON");
+				} else if (key < 1) {
+					throw SerializationError("Can't use zero-based or negative indexes in JSON");
+				} else if (floor(key) != key) {
+					throw SerializationError("Can't use indexes with a fractional part in JSON");
+				}
+				root[(Json::ArrayIndex) key - 1] = value;
+			} else if (keytype == LUA_TSTRING) {
+				if (roottype != Json::nullValue && roottype != Json::objectValue) {
+					throw SerializationError("Can't mix array and object values in JSON");
+				}
+				root[lua_tostring(L, -1)] = value;
+			} else {
+				throw SerializationError("Lua key to convert to JSON is not a string or number");
+			}
+		}
+	} else if (type == LUA_TNIL) {
+		root = Json::nullValue;
+	} else {
+		throw SerializationError("Can only store booleans, numbers, strings, objects, arrays, and null in JSON");
+	}
+	lua_pop(L, 1); // Pop value
+}
+
