@@ -17,13 +17,12 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#include "cpp_api/scriptapi.h"
-#include "common/c_converter.h"
-#include "common/c_content.h"
 #include "lua_api/l_object.h"
-#include "common/c_internal.h"
+#include "lua_api/l_internal.h"
 #include "lua_api/l_inventory.h"
 #include "lua_api/l_item.h"
+#include "common/c_converter.h"
+#include "common/c_content.h"
 #include "log.h"
 #include "tool.h"
 #include "serverobject.h"
@@ -44,7 +43,8 @@ struct EnumString es_HudElementType[] =
 
 struct EnumString es_HudElementStat[] =
 {
-	{HUD_STAT_POS,    "pos"},
+	{HUD_STAT_POS,    "position"},
+	{HUD_STAT_POS,    "pos"}, /* Deprecated, only for compatibility's sake */
 	{HUD_STAT_NAME,   "name"},
 	{HUD_STAT_SCALE,  "scale"},
 	{HUD_STAT_TEXT,   "text"},
@@ -62,6 +62,7 @@ struct EnumString es_HudBuiltinElement[] =
 	{HUD_FLAG_HEALTHBAR_VISIBLE, "healthbar"},
 	{HUD_FLAG_CROSSHAIR_VISIBLE, "crosshair"},
 	{HUD_FLAG_WIELDITEM_VISIBLE, "wielditem"},
+	{HUD_FLAG_BREATHBAR_VISIBLE, "breathbar"},
 	{0, NULL},
 };
 
@@ -274,7 +275,7 @@ int ObjectRef::l_get_inventory(lua_State *L)
 	if(co == NULL) return 0;
 	// Do it
 	InventoryLocation loc = co->getInventoryLocation();
-	if(STACK_TO_SERVER(L)->getInventory(loc) != NULL)
+	if(getServer(L)->getInventory(loc) != NULL)
 		InvRef::create(L, loc);
 	else
 		lua_pushnil(L); // An object may have no inventory (nil)
@@ -329,7 +330,7 @@ int ObjectRef::l_set_wielded_item(lua_State *L)
 	ServerActiveObject *co = getobject(ref);
 	if(co == NULL) return 0;
 	// Do it
-	ItemStack item = read_item(L, 2,STACK_TO_SERVER(L));
+	ItemStack item = read_item(L, 2, getServer(L));
 	bool success = co->setWieldedItem(item);
 	lua_pushboolean(L, success);
 	return 1;
@@ -349,24 +350,35 @@ int ObjectRef::l_set_armor_groups(lua_State *L)
 	return 0;
 }
 
-// set_physics_override(self, physics_override_speed, physics_override_jump, physics_override_gravity)
+// set_physics_override(self, physics_override_speed, physics_override_jump,
+//                      physics_override_gravity, sneak, sneak_glitch)
 int ObjectRef::l_set_physics_override(lua_State *L)
 {
 	ObjectRef *ref = checkobject(L, 1);
 	PlayerSAO *co = (PlayerSAO *) getobject(ref);
 	if(co == NULL) return 0;
 	// Do it
-	if(!lua_isnil(L, 2)){
-		co->m_physics_override_speed = lua_tonumber(L, 2);
-		co->m_physics_override_sent = false;
-	}
-	if(!lua_isnil(L, 3)){
-		co->m_physics_override_jump = lua_tonumber(L, 3);
-		co->m_physics_override_sent = false;
-	}
-	if(!lua_isnil(L, 4)){
-		co->m_physics_override_gravity = lua_tonumber(L, 4);
+	if (lua_istable(L, 2)) {
+		co->m_physics_override_speed = getfloatfield_default(L, 2, "speed", co->m_physics_override_speed);
+		co->m_physics_override_jump = getfloatfield_default(L, 2, "jump", co->m_physics_override_jump);
+		co->m_physics_override_gravity = getfloatfield_default(L, 2, "gravity", co->m_physics_override_gravity);
+		co->m_physics_override_sneak = getboolfield_default(L, 2, "sneak", co->m_physics_override_sneak);
+		co->m_physics_override_sneak_glitch = getboolfield_default(L, 2, "sneak_glitch", co->m_physics_override_sneak_glitch);
 		co->m_physics_override_sent = false;
+	} else {
+		// old, non-table format
+		if(!lua_isnil(L, 2)){
+			co->m_physics_override_speed = lua_tonumber(L, 2);
+			co->m_physics_override_sent = false;
+		}
+		if(!lua_isnil(L, 3)){
+			co->m_physics_override_jump = lua_tonumber(L, 3);
+			co->m_physics_override_sent = false;
+		}
+		if(!lua_isnil(L, 4)){
+			co->m_physics_override_gravity = lua_tonumber(L, 4);
+			co->m_physics_override_sent = false;
+		}
 	}
 	return 0;
 }
@@ -621,6 +633,16 @@ int ObjectRef::l_is_player(lua_State *L)
 	return 1;
 }
 
+// is_player_connected(self)
+int ObjectRef::l_is_player_connected(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	lua_pushboolean(L, (player != NULL && player->peer_id != 0));
+	return 1;
+}
+
 // get_player_name(self)
 int ObjectRef::l_get_player_name(lua_State *L)
 {
@@ -701,6 +723,33 @@ int ObjectRef::l_set_look_yaw(lua_State *L)
 	return 1;
 }
 
+// set_breath(self, breath)
+int ObjectRef::l_set_breath(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	ObjectRef *ref = checkobject(L, 1);
+	PlayerSAO* co = getplayersao(ref);
+	if(co == NULL) return 0;
+	u16 breath = luaL_checknumber(L, 2);
+	// Do it
+	co->setBreath(breath);
+	co->m_breath_not_sent = true;
+	return 0;
+}
+
+// get_breath(self)
+int ObjectRef::l_get_breath(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+	ObjectRef *ref = checkobject(L, 1);
+	PlayerSAO* co = getplayersao(ref);
+	if(co == NULL) return 0;
+	// Do it
+	u16 breath = co->getBreath();
+	lua_pushinteger (L, breath);
+	return 1;
+}
+
 // set_inventory_formspec(self, formspec)
 int ObjectRef::l_set_inventory_formspec(lua_State *L)
 {
@@ -711,7 +760,7 @@ int ObjectRef::l_set_inventory_formspec(lua_State *L)
 	std::string formspec = luaL_checkstring(L, 2);
 
 	player->inventory_formspec = formspec;
-	STACK_TO_SERVER(L)->reportInventoryFormspecModified(player->getName());
+	getServer(L)->reportInventoryFormspecModified(player->getName());
 	lua_pushboolean(L, true);
 	return 1;
 }
@@ -813,7 +862,7 @@ int ObjectRef::l_hud_add(lua_State *L)
 	elem->offset = lua_istable(L, -1) ? read_v2f(L, -1) : v2f();
 	lua_pop(L, 1);
 
-	u32 id = STACK_TO_SERVER(L)->hudAdd(player, elem);
+	u32 id = getServer(L)->hudAdd(player, elem);
 	if (id == (u32)-1) {
 		delete elem;
 		return 0;
@@ -835,7 +884,7 @@ int ObjectRef::l_hud_remove(lua_State *L)
 	if (!lua_isnil(L, 2))
 		id = lua_tonumber(L, 2);
 
-	if (!STACK_TO_SERVER(L)->hudRemove(player, id))
+	if (!getServer(L)->hudRemove(player, id))
 		return 0;
 
 	lua_pushboolean(L, true);
@@ -850,16 +899,18 @@ int ObjectRef::l_hud_change(lua_State *L)
 	if (player == NULL)
 		return 0;
 
-	u32 id = -1;
-	if (!lua_isnil(L, 2))
-		id = lua_tonumber(L, 2);
-
-	HudElementStat stat = (HudElementStat)getenumfield(L, 3, "stat",
-								es_HudElementStat, HUD_STAT_NUMBER);
-
+	u32 id = !lua_isnil(L, 2) ? lua_tonumber(L, 2) : -1;
 	if (id >= player->hud.size())
 		return 0;
 
+	HudElementStat stat = HUD_STAT_NUMBER;
+	if (!lua_isnil(L, 3)) {
+		int statint;
+		std::string statstr = lua_tostring(L, 3);
+		stat = string_to_enum(es_HudElementStat, statint, statstr) ?
+				(HudElementStat)statint : HUD_STAT_NUMBER;
+	}
+
 	void *value = NULL;
 	HudElement *e = player->hud[id];
 	if (!e)
@@ -901,7 +952,7 @@ int ObjectRef::l_hud_change(lua_State *L)
 			value = &e->offset;
 	}
 
-	STACK_TO_SERVER(L)->hudChange(player, id, stat, value);
+	getServer(L)->hudChange(player, id, stat, value);
 
 	lua_pushboolean(L, true);
 	return 1;
@@ -971,7 +1022,7 @@ int ObjectRef::l_hud_set_flags(lua_State *L)
 			mask  |= esp[i].num;
 		}
 	}
-	if (!STACK_TO_SERVER(L)->hudSetFlags(player, flags, mask))
+	if (!getServer(L)->hudSetFlags(player, flags, mask))
 		return 0;
 
 	lua_pushboolean(L, true);
@@ -988,13 +1039,41 @@ int ObjectRef::l_hud_set_hotbar_itemcount(lua_State *L)
 
 	s32 hotbar_itemcount = lua_tonumber(L, 2);
 
-	if (!STACK_TO_SERVER(L)->hudSetHotbarItemcount(player, hotbar_itemcount))
+	if (!getServer(L)->hudSetHotbarItemcount(player, hotbar_itemcount))
 		return 0;
 
 	lua_pushboolean(L, true);
 	return 1;
 }
 
+// hud_set_hotbar_image(self, name)
+int ObjectRef::l_hud_set_hotbar_image(lua_State *L)
+{
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	if (player == NULL)
+		return 0;
+
+	std::string name = lua_tostring(L, 2);
+
+	getServer(L)->hudSetHotbarImage(player, name);
+	return 1;
+}
+
+// hud_set_hotbar_selected_image(self, name)
+int ObjectRef::l_hud_set_hotbar_selected_image(lua_State *L)
+{
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	if (player == NULL)
+		return 0;
+
+	std::string name = lua_tostring(L, 2);
+
+	getServer(L)->hudSetHotbarSelectedImage(player, name);
+	return 1;
+}
+
 ObjectRef::ObjectRef(ServerActiveObject *object):
 	m_object(object)
 {
@@ -1091,12 +1170,15 @@ const luaL_reg ObjectRef::methods[] = {
 	luamethod(ObjectRef, get_luaentity),
 	// Player-only
 	luamethod(ObjectRef, is_player),
+	luamethod(ObjectRef, is_player_connected),
 	luamethod(ObjectRef, get_player_name),
 	luamethod(ObjectRef, get_look_dir),
 	luamethod(ObjectRef, get_look_pitch),
 	luamethod(ObjectRef, get_look_yaw),
 	luamethod(ObjectRef, set_look_yaw),
 	luamethod(ObjectRef, set_look_pitch),
+	luamethod(ObjectRef, get_breath),
+	luamethod(ObjectRef, set_breath),
 	luamethod(ObjectRef, set_inventory_formspec),
 	luamethod(ObjectRef, get_inventory_formspec),
 	luamethod(ObjectRef, get_player_control),
@@ -1107,7 +1189,7 @@ const luaL_reg ObjectRef::methods[] = {
 	luamethod(ObjectRef, hud_get),
 	luamethod(ObjectRef, hud_set_flags),
 	luamethod(ObjectRef, hud_set_hotbar_itemcount),
+	luamethod(ObjectRef, hud_set_hotbar_image),
+	luamethod(ObjectRef, hud_set_hotbar_selected_image),
 	{0,0}
 };
-
-REGISTER_LUA_REF(ObjectRef)
