@@ -57,92 +57,55 @@ std::string getImagePath(std::string path);
 */
 std::string getTexturePath(const std::string &filename);
 
+void clearTextureNameCache();
+
 /*
-	Specifies a texture in an atlas.
-
-	This is used to specify single textures also.
-
-	This has been designed to be small enough to be thrown around a lot.
+	ITextureSource::generateTextureFromMesh parameters
 */
-struct AtlasPointer
+namespace irr {namespace scene {class IMesh;}}
+struct TextureFromMeshParams
 {
-	u32 id; // Texture id
-	video::ITexture *atlas; // Atlas in where the texture is
-	v2f pos; // Position in atlas
-	v2f size; // Size in atlas
-	u16 tiled; // X-wise tiling count. If 0, width of atlas is width of image.
-
-	AtlasPointer():
-		id(0),
-		atlas(NULL),
-		pos(0,0),
-		size(1,1),
-		tiled(1)
-	{}
-
-	AtlasPointer(
-			u16 id_,
-			video::ITexture *atlas_=NULL,
-			v2f pos_=v2f(0,0),
-			v2f size_=v2f(1,1),
-			u16 tiled_=1
-		):
-		id(id_),
-		atlas(atlas_),
-		pos(pos_),
-		size(size_),
-		tiled(tiled_)
-	{
-	}
-
-	bool operator==(const AtlasPointer &other) const
-	{
-		return (
-			id == other.id
-		);
-		/*return (
-			id == other.id &&
-			atlas == other.atlas &&
-			pos == other.pos &&
-			size == other.size &&
-			tiled == other.tiled
-		);*/
-	}
-
-	bool operator!=(const AtlasPointer &other) const
-	{
-		return !(*this == other);
-	}
-
-	float x0(){ return pos.X; }
-	float x1(){ return pos.X + size.X; }
-	float y0(){ return pos.Y; }
-	float y1(){ return pos.Y + size.Y; }
+	scene::IMesh *mesh;
+	core::dimension2d<u32> dim;
+	std::string rtt_texture_name;
+	bool delete_texture_on_shutdown;
+	v3f camera_position;
+	v3f camera_lookat;
+	core::CMatrix4<f32> camera_projection_matrix;
+	video::SColorf ambient_light;
+	v3f light_position;
+	video::SColorf light_color;
+	f32 light_radius;
 };
 
 /*
 	TextureSource creates and caches textures.
 */
 
-class ITextureSource
+class ISimpleTextureSource
+{
+public:
+	ISimpleTextureSource(){}
+	virtual ~ISimpleTextureSource(){}
+	virtual video::ITexture* getTexture(
+			const std::string &name, u32 *id = NULL) = 0;
+};
+
+class ITextureSource : public ISimpleTextureSource
 {
 public:
 	ITextureSource(){}
 	virtual ~ITextureSource(){}
-	virtual u32 getTextureId(const std::string &name){return 0;}
-	virtual u32 getTextureIdDirect(const std::string &name){return 0;}
-	virtual std::string getTextureName(u32 id){return "";}
-	virtual AtlasPointer getTexture(u32 id){return AtlasPointer(0);}
-	virtual AtlasPointer getTexture(const std::string &name)
-		{return AtlasPointer(0);}
-	virtual video::ITexture* getTextureRaw(const std::string &name)
-		{return NULL;}
-	virtual AtlasPointer getTextureRawAP(const AtlasPointer &ap)
-		{return AtlasPointer(0);}
-	virtual IrrlichtDevice* getDevice()
-		{return NULL;}
-	virtual void updateAP(AtlasPointer &ap){};
+	virtual u32 getTextureId(const std::string &name)=0;
+	virtual u32 getTextureIdDirect(const std::string &name)=0;
+	virtual std::string getTextureName(u32 id)=0;
+	virtual video::ITexture* getTexture(u32 id)=0;
+	virtual video::ITexture* getTexture(
+			const std::string &name, u32 *id = NULL)=0;
+	virtual IrrlichtDevice* getDevice()=0;
 	virtual bool isKnownSourceImage(const std::string &name)=0;
+	virtual video::ITexture* generateTextureFromMesh(
+			const TextureFromMeshParams &params)=0;
 };
 
 class IWritableTextureSource : public ITextureSource
@@ -150,23 +113,20 @@ class IWritableTextureSource : public ITextureSource
 public:
 	IWritableTextureSource(){}
 	virtual ~IWritableTextureSource(){}
-	virtual u32 getTextureId(const std::string &name){return 0;}
-	virtual u32 getTextureIdDirect(const std::string &name){return 0;}
-	virtual std::string getTextureName(u32 id){return "";}
-	virtual AtlasPointer getTexture(u32 id){return AtlasPointer(0);}
-	virtual AtlasPointer getTexture(const std::string &name)
-		{return AtlasPointer(0);}
-	virtual video::ITexture* getTextureRaw(const std::string &name)
-		{return NULL;}
-	virtual IrrlichtDevice* getDevice()
-		{return NULL;}
-	virtual void updateAP(AtlasPointer &ap){};
+	virtual u32 getTextureId(const std::string &name)=0;
+	virtual u32 getTextureIdDirect(const std::string &name)=0;
+	virtual std::string getTextureName(u32 id)=0;
+	virtual video::ITexture* getTexture(u32 id)=0;
+	virtual video::ITexture* getTexture(
+			const std::string &name, u32 *id = NULL)=0;
+	virtual IrrlichtDevice* getDevice()=0;
 	virtual bool isKnownSourceImage(const std::string &name)=0;
+	virtual video::ITexture* generateTextureFromMesh(
+			const TextureFromMeshParams &params)=0;
 
 	virtual void processQueue()=0;
 	virtual void insertSourceImage(const std::string &name, video::IImage *img)=0;
 	virtual void rebuildImagesAndTextures()=0;
-	virtual void buildMainAtlas(class IGameDef *gamedef)=0;
 };
 
 IWritableTextureSource* createTextureSource(IrrlichtDevice *device);
@@ -176,6 +136,8 @@ enum MaterialType{
 	TILE_MATERIAL_ALPHA,
 	TILE_MATERIAL_LIQUID_TRANSPARENT,
 	TILE_MATERIAL_LIQUID_OPAQUE,
+	TILE_MATERIAL_LEAVES,
+	TILE_MATERIAL_PLANTS
 };
 
 // Material flags
@@ -189,8 +151,6 @@ enum MaterialType{
 // Animation made up by splitting the texture to vertical frames, as
 // defined by extra parameters
 #define MATERIAL_FLAG_ANIMATION_VERTICAL_FRAMES 0x08
-// Whether liquid shader should be used
-#define MATERIAL_FLAG_
 
 /*
 	This fully defines the looks of a tile.
@@ -199,7 +159,8 @@ enum MaterialType{
 struct TileSpec
 {
 	TileSpec():
-		texture(0),
+		texture_id(0),
+		texture(NULL),
 		alpha(255),
 		material_type(TILE_MATERIAL_BASIC),
 		material_flags(
@@ -207,14 +168,16 @@ struct TileSpec
 			MATERIAL_FLAG_BACKFACE_CULLING
 		),
 		animation_frame_count(1),
-		animation_frame_length_ms(0)
+		animation_frame_length_ms(0),
+		rotation(0)
 	{
 	}
 
 	bool operator==(const TileSpec &other) const
 	{
 		return (
-			texture == other.texture &&
+			texture_id == other.texture_id &&
+			/* texture == other.texture && */
 			alpha == other.alpha &&
 			material_type == other.material_type &&
 			material_flags == other.material_flags &&
@@ -243,13 +206,21 @@ struct TileSpec
 		case TILE_MATERIAL_LIQUID_OPAQUE:
 			material.MaterialType = video::EMT_SOLID;
 			break;
+		case TILE_MATERIAL_LEAVES:
+			material.MaterialType = video::EMT_TRANSPARENT_ALPHA_CHANNEL_REF;
+			break;
+		case TILE_MATERIAL_PLANTS:
+			material.MaterialType = video::EMT_TRANSPARENT_ALPHA_CHANNEL_REF;
+		break;
 		}
 		material.BackfaceCulling = (material_flags & MATERIAL_FLAG_BACKFACE_CULLING) ? true : false;
 	}
 	void applyMaterialOptionsWithShaders(video::SMaterial &material,
 			const video::E_MATERIAL_TYPE &basic,
 			const video::E_MATERIAL_TYPE &liquid,
-			const video::E_MATERIAL_TYPE &alpha) const
+			const video::E_MATERIAL_TYPE &alpha,
+			const video::E_MATERIAL_TYPE &leaves,
+			const video::E_MATERIAL_TYPE &plants) const
 	{
 		switch(material_type){
 		case TILE_MATERIAL_BASIC:
@@ -264,18 +235,18 @@ struct TileSpec
 		case TILE_MATERIAL_LIQUID_OPAQUE:
 			material.MaterialType = liquid;
 			break;
+		case TILE_MATERIAL_LEAVES:
+			material.MaterialType = leaves;
+			break;
+		case TILE_MATERIAL_PLANTS:
+			material.MaterialType = plants;
+			break;
 		}
 		material.BackfaceCulling = (material_flags & MATERIAL_FLAG_BACKFACE_CULLING) ? true : false;
 	}
 	
-	// NOTE: Deprecated, i guess?
-	void setTexturePos(u8 tx_, u8 ty_, u8 tw_, u8 th_)
-	{
-		texture.pos = v2f((float)tx_/256.0, (float)ty_/256.0);
-		texture.size = v2f(((float)tw_ + 1.0)/256.0, ((float)th_ + 1.0)/256.0);
-	}
-	
-	AtlasPointer texture;
+	u32 texture_id;
+	video::ITexture *texture;
 	// Vertex alpha (when MATERIAL_ALPHA_VERTEX is used)
 	u8 alpha;
 	// Material parameters
